      PROGRAM EGRUN
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH GENERALIZED RUNS PERMUTATION
C         PROCEDURES (EGRUN) IS CALCULATED IN THIS FORTRAN-77 PROGRAM.
C         THIS EXACT TEST IS DESIGNED TO ANALYZE GENERALIZED RUNS DATA
C         CONSISTING OF POSITIONS WITHIN G A PRIORI DISJOINT GROUPS OF
C         OBJECTS AND A POSSIBLY EMPTY EXCESS GROUP IN A FINITE POPULATION
C         OF N OBJECTS.  THE PRESENT MAXIMUM VALUES OF G AND N IN THIS
C         PROGRAM ARE 10 AND 30, RESPECTIVELY.
C
C         PROGRAM MODIFIED 8/15/99
C
C       FIRST LINE OF DATA FILE:  PROBLEM HEADING.
C
C       SECOND LINE:  N,CFORM,G
C       THIRD LINE:  n(1),...,n(G)
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = n(I)/sum(n(I))
C            CFORM = 2:  C(I) = (n(I)-1)/sum(n(I)-1)
C            CFORM = 3:  C(I) = 1/sum(1)
C            CFORM = 4:  C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))
C       CFORM IS PASSED AS THE SECOND ENTRY ON THE SECOND LINE IN
C       THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED IN
C       SUBROUTINE DIST AND STORED IN ARRAY C(I).
C
C       (2N-I)(I-1)/2+J-I+3RD LINE:  DEL(I,J) WHERE 0 < I < J < N+1.
C       THUS THE FOURTH THROUGH THE N(N-1)/2+3RD LINES ARE DEL(1,2)
C       THROUGH DEL(N-1,N), RESPECTIVELY.  DEL(I,J) IS 1 IF THE Ith
C       AND Jth DISTINCT OBJECTS ARE LINKED AND IS 0 OTHERWISE.
C
      DOUBLE PRECISION  D(450),ONE,ZERO,
     +   S(10),C(10),PROB,DELTA1,Y
      CHARACTER  FIN*40,FOUT*40,HEAD*75
      INTEGER  G,B(11),CFORM
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER FILE NAME FOR INPUT DATA'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,100) FOUT
      OPEN(1,FILE=FIN)
      READ(1,100) HEAD
      READ(1,*) N,CFORM,G
      READ(1,*) (B(I),I = 1,G)
      NT = N*(N-1)/2
      READ(1,*,END=200) (D(I),I = 1,NT)
      GO TO 250
  200 WRITE(*,205) 'ERROR: END-OF-FILE ENCOUNTERED'
  205 FORMAT(/,T3,A)
      STOP
  250 CLOSE(1)
      OPEN(2,FILE=FOUT,STATUS='NEW')
      WRITE(2,100) HEAD
      WRITE(2,330)
  330 FORMAT(T25,'*** OUTPUT OF EGRUN ANALYSIS ***',/)
      WRITE(2,340) N,G
  340 FORMAT(T5,'NUMBER OF OBJECTS =',I3,/,
     +   T5,'NUMBER OF GROUPS =',I3)
      GO TO (341,342,343,344) CFORM
  341 WRITE(2,346) 'C(I) = n(I)/sum(n(I))'
      GO TO 348
  342 WRITE(2,346) 'C(I) = (n(I)-1)/sum(n(I)-1)'
      GO TO 348
  343 WRITE(2,346) 'C(I) = 1/sum(1)'
      GO TO 348
  344 WRITE(2,346) 'C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))'
  346 FORMAT(T5,A,/)
  348 ZERO = 0.0D0
      ONE = 1.0D0
      K = 0
      DO 350 I = 1,G
         WRITE(2,360) I,B(I)
         K = K+B(I)
  350 CONTINUE
  360 FORMAT(T5,'SIZE OF GROUP ',I2,' =',I3,/)
      B(G+1) = N-K
      DO 370 I = 1,G
         S(I) = ONE*B(I)*(B(I)-1)/2
  370 CONTINUE
      WRITE(2,380) B(G+1)
  380 FORMAT(T5,'SIZE OF EXCESS GROUP =',I3,2(/))
      GO TO (410,430,450,470) CFORM
  410 DO 420 I = 1,G
         C(I) = ONE*B(I)/K
  420 CONTINUE
      GO TO 500
  430 DO 440 I = 1,G
         C(I) = ONE*(B(I)-1)/(K-G)
  440 CONTINUE
      GO TO 500
  450 DO 460 I = 1,G
         C(I) = ONE/G
  460 CONTINUE
      GO TO 500
  470 Y = ZERO
      DO 475 I = 1,G
         Y = Y+ONE*B(I)*(B(I)-1)
  475 CONTINUE
      DO 480 I = 1,G
         C(I) = ONE*B(I)*(B(I)-1)/Y
  480 CONTINUE
  500 CALL PERM(N,G,B,S,C,D,KOUNT1,KOUNT2,DELTA1)
      PROB = ONE*KOUNT2/KOUNT1
      WRITE(2,590) DELTA1
  590 FORMAT(T5,'OBSERVED GRUN STATISTIC =',G15.8)
      WRITE(2,600) PROB
  600 FORMAT(T5,'EXACT GRUN P-VALUE =',G15.8,/)
      CLOSE(2)
      END
C
C
      SUBROUTINE PERM(N,G,B,S,C,D,KOUNT1,KOUNT2,DELTA1)
C
      DOUBLE PRECISION  D(450),S(10),C(10),DELTA1
      INTEGER  G,A(11,30),B(11),P(30)
C
      K = G
      IF(B(G+1).NE.0) K = K+1
      L = N
      DO 50 I = 1,K-1
         DO 30 J = 1,B(I)
            A(I,J) = I
   30    CONTINUE
         DO 40 J = B(I)+1,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-B(I)
   50 CONTINUE
      KOUNT1 = 0
      KOUNT2 = 0
   60 I = K-1
      NMARKS = B(K)+B(I)
      IF(KOUNT1.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+B(I)
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL DVAL(N,G,B,S,C,D,P,KOUNT1,KOUNT2,DELTA1)
      KOUNT1 = KOUNT1+1
      GO TO 60
      END
C
C
      SUBROUTINE DVAL(N,G,B,S,C,D,P,KOUNT1,KOUNT2,DELTA1)
C
      DOUBLE PRECISION  D(450),S(10),C(10),ZERO,ONE,XI(10),DELTA,DELTA1,
     +   PREC
      INTEGER  G,B(11),P(30)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PREC = 1.0D-12
      LL = 0
      DO 30 II = 1,G
         KK = LL+1
         LL = LL+B(II)
         XI(II) = ZERO
         DO 20 III = KK,LL-1
            I = P(III)
            DO 10 JJJ = III+1,LL
               J = P(JJJ)
               K = N*(I-1)+J-(I*(I+1))/2
               XI(II) = XI(II)+D(K)
   10       CONTINUE
   20    CONTINUE
         XI(II) = XI(II)/S(II)
   30 CONTINUE
      DELTA = ZERO
      DO 40 II = 1,G
         DELTA = DELTA+C(II)*XI(II)
   40 CONTINUE
      IF(KOUNT1.EQ.0) DELTA1 = DELTA*(ONE-PREC)
      IF(DELTA.GT.DELTA1) KOUNT2 = KOUNT2+1
      RETURN
      END
