      PROGRAM GRUN
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH GENERALIZED RUNS PERMUTATION
C         PROCEDURES (GRUN) IS CALCULATED IN THIS FORTRAN-77 PROGRAM.
C         THIS TEST IS DESIGNED TO ANALYZE GENERALIZED RUNS DATA
C         CONSISTING OF POSITIONS WITHIN G A PRIORI DISJOINT GROUPS
C         OF OBJECTS AND A POSSIBLY EMPTY EXCESS GROUP IN A FINITE
C         POPULATION OF N OBJECTS.  THE PRESENT MAXIMUM VALUES OF
C         G AND N IN THIS PROGRAM ARE 40 AND 200, RESPECTIVELY.
C
C         PROGRAM MODIFIED 8/14/97
C
C       FIRST LINE OF DATA FILE:  PROBLEM HEADING.
C
C       SECOND LINE:  N,CFORM,G
C       THIRD LINE:  n(1),...,n(G)
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = n(I)/sum(n(I))
C            CFORM = 2:  C(I) = (n(I)-1)/sum(n(I)-1)
C            CFORM = 3:  C(I) = 1/sum(1)
C            CFORM = 4:  C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))
C       CFORM IS PASSED AS THE SECOND ENTRY ON THE SECOND LINE IN
C       THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED IN
C       SUBROUTINE DIST AND STORED IN ARRAY C(I).
C
C       (2N-I)(I-1)/2+J-I+3RD LINE:  DEL(I,J) WHERE 0 < I < J < N+1.
C       THUS THE FOURTH THROUGH THE N(N-1)/2+3RD LINES ARE DEL(1,2)
C       THROUGH DEL(N-1,N), RESPECTIVELY.  DEL(I,J) IS 1 IF THE Ith
C       AND Jth DISTINCT OBJECTS ARE LINKED AND IS 0 OTHERWISE.
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  C(40),DEL(200,200),ITHGRP(40),XI(40)
      INTEGER  CFORM
C
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,1000) FIN
 1000 FORMAT(A)
      OPEN(3,FILE=FIN)
      READ(3,1000) HEAD
      READ(3,*) NCASES,CFORM,NGRPS
      IF(NCASES.LT.6) STOP 'TOTAL NUMBER OF CASES < 6'
      READ(3,*) (ITHGRP(I),I = 1,NGRPS)
      DO 1010 I = 1,NCASES
         DEL(I,I) = ZERO
 1010 CONTINUE
      DO 1060 I = 1,NCASES-1
         DO 1050 J = I+1,NCASES
            READ(3,*,END=1070) DEL(I,J)
            DEL(J,I) = DEL(I,J)
 1050    CONTINUE
 1060 CONTINUE
      CLOSE(3)
      GO TO 1080
 1070 WRITE(*,1075) 'ERROR:  END-OF FILE ENCOUNTERED'
 1075 FORMAT(/,T3,A)
      STOP
 1080 WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1000) FOUT
      CALL DIST(NCASES,CFORM,NGRPS,ITHGRP,DEL,C,XI,
     +   DBAR,D1,VAR,T,GAM1,PROB)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,1000) HEAD
      HEAD = 'GENERALIZED RUNS PERMUTATION PROCEDURES (GRUN)'
      WRITE(6,1100) HEAD
 1100 FORMAT(/,T23,A,/)
      WRITE(6,1200) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1200 FORMAT(T10,A,I3,A)
      WRITE(6,1300) 'IN: ', NGRPS, ' GROUPS OF SIZES ',
     +              (ITHGRP(I),I = 1,NGRPS)
 1300 FORMAT(T10,A,I5,A,10I5)
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = n(I)/sum(n(I))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = (n(I)-1)/sum(n(I)-1)'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = 1/sum(1)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))'
 1450 FORMAT(T10,A)
 1500 WRITE(6,1000) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZE ', ITHGRP(I),
     +                 ' HAS AN AVERAGE DISTANCE =', XI(I)
 1600    FORMAT(T10,A,I2,A,I3,A,G16.8)
 1700 CONTINUE
      WRITE(6,1000) ' '
      WRITE(6,1800) 'THE TEST STATISTIC IS =', T
 1800 FORMAT(T10,A,G16.8)
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
      WRITE(6,1800) 'THE EXPECTED DELTA IS =', D1
      WRITE(6,1800) 'THE VARIANCE OF DELTA =', VAR
      WRITE(6,1800) 'THE SKEWNESS OF DELTA =', GAM1
      WRITE(6,1850) 'P-VALUE OF A LARGER OR EQUAL DELTA =', PROB
 1850 FORMAT(/,T10,A,G16.8)
      WRITE(6,2000) '* * *   G R U N   E N D   * * *'
 2000 FORMAT(/,T22,A)
      END
C
C
      SUBROUTINE DIST(NCASES,CFORM,NGRPS,ITHGRP,DEL,C,XI,
     +   DBAR,D1,VAR,T,GAM1,PROB)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  B(12),C(40),DC(3,200),DEL(200,200),
     +   ITHGRP(40),Q(6),R(40,6),S(11),XI(40)
      INTEGER  CFORM
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DBAR = ZERO
      DO 1000 I = 1,NGRPS
         XI(I) = ZERO
 1000 CONTINUE
      L1 = ZERO
      L2 = ZERO
      DO 1400 I = 1,NGRPS
         L1 = L2
         L2 = L2+ITHGRP(I)
         DO 1300 J = L1+1,L2-1
            DO 1200 K = J+1,L2
               XI(I) = XI(I)+DEL(J,K)
 1200       CONTINUE
 1300    CONTINUE
 1400 CONTINUE
      GO TO (1410,1430,1450,1470) CFORM
 1410 FK = ZERO
      DO 1415 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1415 CONTINUE
      DO 1420 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)/FK
 1420 CONTINUE
      GO TO 1490
 1430 FK = ZERO
      DO 1435 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1435 CONTINUE
      DO 1440 I = 1,NGRPS
         C(I) = ONE*(ITHGRP(I)-1)/(FK-NGRPS)
 1440 CONTINUE
      GO TO 1490
 1450 DO 1460 I = 1,NGRPS
         C(I) = ONE/NGRPS
 1460 CONTINUE
      GO TO 1490
 1470 Y = ZERO
      DO 1475 I = 1,NGRPS
         Y = Y+ONE*ITHGRP(I)*(ITHGRP(I)-1)
 1475 CONTINUE
      DO 1480 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)*(ITHGRP(I)-1)/Y
 1480 CONTINUE
 1490 DO 1500 I = 1,NGRPS
         XI(I) = XI(I)*2/(ITHGRP(I)*(ITHGRP(I)-1))
         DBAR = DBAR+C(I)*XI(I)
 1500 CONTINUE
      DO 2150 I = 1,NCASES
         DO 2140 K = 1,3
            DC(K,I) = ZERO
            DO 2130 J = 1,NCASES
               DC(K,I) = DC(K,I)+DEL(I,J)**K
 2130       CONTINUE
 2140    CONTINUE
 2150 CONTINUE
      DO 2160 I = 1,12
         B(I) = ZERO
 2160 CONTINUE
      DO 2170 I = 1,NCASES
         B(1) = B(1)+DC(1,I)
         B(2) = B(2)+DC(2,I)
         B(3) = B(3)+DC(1,I)**2
         B(5) = B(5)+DC(3,I)
         B(6) = B(6)+DC(1,I)*DC(2,I)
         B(10) = B(10)+DC(1,I)**3
 2170 CONTINUE
      DO 2190 I = 1,NCASES-1
         DO 2180 J = I+1,NCASES
            B(9) = B(9)+DC(1,I)*DC(1,J)*DEL(I,J)
 2180    CONTINUE
 2190 CONTINUE
      B(9) = B(9)*2
      DO 2240 I = 1,NCASES-2
         DO 2230 J = I+1,NCASES-1
            DO 2220 K = J+1,NCASES
               SUM = DEL(I,J)*DEL(I,K)*DEL(J,K)
               B(7) = B(7)+SUM
 2220       CONTINUE
 2230    CONTINUE
 2240 CONTINUE
      B(7) = B(7)*6
      B(4) = B(1)**2
      B(8) = B(1)*B(2)
      B(11) = B(1)*B(3)
      B(12) = B(1)**3
      A1 = B(1)
      A2 = B(2)
      A3 = B(3)-B(2)
      A4 = B(4)-A2*2-A3*4
      A5 = B(5)
      A6 = B(6)-B(5)
      A7 = B(7)
      A8 = B(8)+(B(5)-B(6)*2)*2
      A9 = B(9)+B(5)-B(6)*2-B(7)
      A10 = B(10)+B(5)*2-B(6)*3
      A11 = B(11)-(B(10)-B(7)+(B(5)+B(9))*2)*2+B(6)*10-B(8)
      A12 = B(12)-(A5+(A7+A10+(A6+A9)*3)*2)*4-(A8+A11*2)*6
      Q(1) = NCASES
      DO 2280 I = 2,6
         Q(I) = Q(I-1)*(NCASES-I+1)
 2280 CONTINUE
      D1 = A1/Q(2)
      D2 = A2/Q(2)
      D3 = A3/Q(3)
      D4 = A4/Q(4)
      D5 = A5/Q(2)
      D6 = A6/Q(3)
      D7 = A7/Q(3)
      D8 = A8/Q(4)
      D9 = A9/Q(4)
      D10 = A10/Q(4)
      D11 = A11/Q(5)
      D12 = A12/Q(6)
      DO 2300 I = 1,NGRPS
         R(I,1) = ITHGRP(I)
         DO 2290 J = 2,6
            R(I,J) = R(I,J-1)*(ITHGRP(I)-J+1)
 2290    CONTINUE
 2300 CONTINUE
      DO 2310 I = 1,11
         S(I) = ZERO
 2310 CONTINUE
      DO 2340 I = 1,NGRPS
         S(4) = S(4)+C(I)**2/R(I,2)
         S(5) = S(5)+C(I)**2/R(I,1)
         S(6) = S(6)+C(I)**3/R(I,2)**2
         S(7) = S(7)+C(I)**3*R(I,3)/R(I,2)**3
         S(8) = S(8)+C(I)**2*((ONE-C(I))/R(I,2)+C(I)*R(I,4)/R(I,2)**3)
         S(9) = S(9)+C(I)**3*R(I,4)/R(I,2)**3
         S(10) = S(10)+C(I)**2*((ONE-C(I))*R(I,3)/R(I,2)**2+
     1           C(I)*R(I,5)/R(I,2)**3)
         S(11) = S(11)+C(I)*((ONE-C(I))*(ONE-C(I)*2)+C(I)*(ONE-C(I))*
     1           R(I,4)*3/R(I,2)**2+C(I)**2*R(I,6)/R(I,2)**3)
 2340 CONTINUE
      VAR = (S(4)-ONE/Q(2))*(D2-D3*2+D4)*2+(S(5)-ONE/Q(1))*(D3-D4)*4
      STDEV = DSQRT(VAR)
      T = (DBAR-D1)/STDEV
      EDELT3 = S(6)*D5*4+S(7)*(D6*3+D7)*8+S(8)*D8*6+
     1         S(9)*(D9*3+D10)*8+S(10)*D11*12+S(11)*D12
      CUM3 = EDELT3-D1*VAR*3-D1**3
      GAM1 = CUM3/STDEV**3
      CALL PVALUE(T,GAM1,PROB)
      RETURN
      END
C
C
      SUBROUTINE PVALUE(Z,GAM,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE
C        OF Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE
C        OF Z.
C
      DOUBLE PRECISION  Z,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H2,H3,H4
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      F = R*R+10
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)-
     +    ONE/(F*F*F*360)
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      A  = R*R-1
      B  = R*R*(DLOG(R)-1)-U
      W  = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(Z.LT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = Z
      Y = Z+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(I*2-1))-R*(X+G1*(I*2-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H4 = E(G)
      PROB = G3*(H0+(H1+H4)*4+H2*2+H3)
      RETURN
   30 IF(Z.GT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = Z-15
      Y = Z
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(I*2-1))+R*(X+G1*(I*2-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H4 = E(G)
      PROB = ONE-G3*(H0+(H1+H4)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 60
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
