      PROGRAM OLSREG
      IMPLICIT REAL*8(A-H,O-Z)
C
      CHARACTER  HEADA*70,HEADC*70,FINA*40,FINC*40,FOUT*40
      DIMENSION  A0(250,51),C0(250,51),BA(50,50),BC(50,50),XA(50),XC(50)
C
C    PROGRAM OLSREG PROVIDES A P-VALUE FOR TESTING A NESTED NULL MODEL
C    RELATIVE TO A FULL ALTERNATIVE MODEL ASSUMING NORMALITY OF ERRORS
C
C          PRESENT SIZE LIMITS ON PROGRAM OLSREG ARE THE FOLLOWING:
C           M = 250 OBSERVED CASES & N = 50 INDEPENDENT VARIABLES
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,10) '*** PROGRAM OLSREG ***'
   10 FORMAT(T19,A)
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA FOR NULL MODEL:'
      READ(*,12) FINA
   12 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA FOR ALTERNATIVE MODEL:'
      READ(*,12) FINC
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,12) FOUT
      OPEN(4,FILE=FINA)
      OPEN(5,FILE=FINC)
      OPEN(7,FILE=FOUT,STATUS='NEW')
C
      READ(4,12) HEADA
      READ(4,*) MA,NA
      NA1 = NA+1
      READ(4,*) ((A0(I,J),J = 1,NA1),I = 1,MA)
      CLOSE(4)
      READ(5,12) HEADC
      READ(5,*) MC,NC
      NC1 = NC+1
      READ(5,*) ((C0(I,J),J = 1,NC1),I = 1,MC)
      CLOSE(5)
      IF(MA.EQ.MC) THEN
         M = MA
      ELSE
         STOP 'SAMPLE SIZES OF NULL AND ALTERNATIVE MODELS NOT EQUAL'
      END IF
C
      NN = NC-NA
      NE = M-NC
      DN = ONE*(NC-NA)
      DA = ONE*(M-NA)
      DC = ONE*(M-NC)
      WRITE(7,1030)
 1030 FORMAT(T20,'*** OUTPUT OF OLSREG ANALYSIS ***')
      WRITE(7,1035) HEADA
 1035 FORMAT(T2,'NULL DATA: ',A)
      WRITE(7,1040) HEADC
 1040 FORMAT(T2,'ALTERNATIVE DATA: ',A)
      IF(NA.EQ.0) THEN
         SA = ZERO
         DO 1060 I = 1,M
            SA = SA+A0(I,1)**2
 1060    CONTINUE
         EA = SA/DA
         GO TO 1120
      END IF
      WRITE(7,1080)
 1080 FORMAT(T2,'NULL HYPOTHESIS REGRESSION COEFFICIENTS',
     +            ' & STANDARD ERRORS:')
      CALL R2(M,NA,SA,XA,BA,A0)
      EA = SA/DA
      DO 1100 I = 1,NA
         SE = DSQRT(BA(I,I)*EA)
         WRITE(7,1090) 'REGRESSION COEFFICIENT ',I,' = ',XA(I),
     +      '& STANDARD ERROR = ',SE
 1090    FORMAT(T4,A,I2,A,G15.8,A,G15.8)
 1100 CONTINUE
 1120 WRITE(7,1150)
 1150 FORMAT(/,T2,'ALTERNATIVE HYPOTHESIS REGRESSION COEFFICIENTS',
     +            ' & STANDARD ERRORS:')
      CALL R2(M,NC,SC,XC,BC,C0)
      EC = SC/DC
      SN = SA-SC
      EN = SN/DN
      DO 1200 I = 1,NC
         SE = DSQRT(BC(I,I)*EC)
         WRITE(7,1190) 'REGRESSION COEFFICIENT ',I,' = ',XC(I),
     +      '& STANDARD ERROR = ',SE
 1190    FORMAT(T4,A,I2,A,G15.8,A,G15.8)
 1200 CONTINUE
C
      WRITE(7,2000) ' '
 2000 FORMAT(T4,A)
      WRITE(7,2100) 'NUMBER OF RESPONSE MEASUREMENTS = ',M
 2100 FORMAT(T4,A,I3)
      WRITE(7,2100) 'NUMBER OF NULL HYPOTHESIS PARAMETERS = ',NA
      WRITE(7,2100) 'NUMBER OF ATERNATIVE HYPOTHESIS PARAMETERS = ',NC
      WRITE(7,2100) 'NUMBER OF TREATMENT DEGREES OF FREEDOM = ',NN
      WRITE(7,2100) 'NUMBER OF ERROR DEGREES OF FREEDOM = ',NE
      WRITE(7,2200) 'NULL HYPOTHESIS MEAN SQUARE ERROR = ',EA
 2200 FORMAT(T4,A,G16.8)
      WRITE(7,2200) 'ALTERNATIVE HYPOTHESIS MEAN SQUARE ERROR = ',EC
      WRITE(7,2200) 'DIFFERENCE MEAN SQUARE ERROR = ',EN
      WRITE(7,2200) 'RESIDUAL SS UNDER NULL MODEL = ',SA
      WRITE(7,2200) 'RESIDUAL SS UNDER ALTERNATIVE MODEL = ',SC
      WRITE(7,2200) 'DIFFERENCE IN RESIDUAL SS = ',SN
C
      FSTAT = (SA-SC)*DC/SC/DN
      IF(FSTAT.LT.1.0D-5) THEN
         PVALUE = ONE
         GO TO 2400
      END IF
C
      CALL FPV(FSTAT,DN,DC,PVALUE)
C
 2400 WRITE(7,2200) 'VALUE OF OLS TEST STATISTIC = ',FSTAT
      WRITE(7,2200) 'P-VALUE FOR NESTED OLS TEST = ',PVALUE
      END
C
C
      SUBROUTINE R2(M,N,RSS,X,B,A)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  A(250,51),X(50),E(250),B(50,50),C(50),Y(250),YH(250),
     +           KP(50),KR(50),KC(50)
C
      ZERO = 0.0D0
      ND =  50
      DO 100 I = 1,M
         Y(I) = A(I,N+1)
  100 CONTINUE
      DO 300 I = 1,N
         C(I) = ZERO
         DO 200 K  = 1,M
            C(I) = C(I)+Y(K)*A(K,I)
  200    CONTINUE
  300 CONTINUE
      DO 600 I = 1,N
         DO 500 J = I,N
            B(I,J) = ZERO
            DO 400 K = 1,M
               B(I,J) = B(I,J)+A(K,I)*A(K,J)
  400       CONTINUE
            B(J,I) = B(I,J)
  500    CONTINUE
  600 CONTINUE
C
      CALL INVERT(N,B,KP,KR,KC,ND)
C
      DO 800 I = 1,N
         X(I) = ZERO
         DO 700 J = 1,N
            X(I) = X(I)+C(J)*B(I,J)
  700    CONTINUE
  800 CONTINUE
      RSS = ZERO
      DO 2300 I = 1,M
         YH(I) = ZERO
         DO 2200 J = 1,N
            YH(I) = YH(I)+X(J)*A(I,J)
 2200    CONTINUE
         E(I) = Y(I)-YH(I)
         RSS = RSS+E(I)**2
 2300 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE INVERT(N,W,KP,KR,KC,ND)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  KP(N),KR(N),KC(N),W(ND,N)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IR = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IR.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IR,L)
               W(IR,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KR(I) = IR
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KR(J).NE.KC(J)) THEN
            K = KR(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE FPV(X,DN,DD,P)
      DOUBLE PRECISION  ONE,X,Y,DN,DD,A,B,P,BETAIN
C
      ONE = 1.0D0
      A = DN/2
      B = DD/2
      Y = A*X/(A*X+B)
      P = ONE - BETAIN(Y,A,B)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION BETAIN(X, A, B)
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL INDEX
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A + B
      CX = ONE - X
      IF (A .GE. AB * X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB + CX * AB)
      RX = XX / CX
   30 TEMP = BB - AI
      IF (NS .EQ. 0) RX = XX
   40 TERM = TERM * TEMP * RX / (AA + AI)
      BETAIN = BETAIN + TERM
      TEMP = DABS(TERM)
      IF (TEMP .LE. TOL .AND. TEMP .LE. TOL * BETAIN) GO TO 50
      AI = AI + 1
      NS = NS - 1
      IF (NS .GE. 0) GO TO 30
      TEMP = AB
      AB = AB + 1
      GO TO 40
   50 BETAIN = BETAIN * DEXP(AA * DLOG(XX) + (BB - 1) * DLOG(CX) -
     +         DLNBTA(AA, BB)) / AA
      IF (INDEX) BETAIN = ONE - BETAIN
      END
C
      DOUBLE PRECISION FUNCTION DLNBTA(A, B)
      DOUBLE PRECISION  A,B,DLNGAM
      DLNBTA = DLNGAM(A) + DLNGAM(B) - DLNGAM(A + B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
      ONE  = 1.0D0
      Z  = X - 1
      D  = ONE
      PI = DATAN(ONE)*4
      F  = Z + 10
      DO 10 J = 1, 9
         D = D * (Z + J)
   10 CONTINUE
      DLNGAM = (F * 2 - 1) * DLOG(F) / 2 - F + DLOG(PI * 2) / 2
     +          - DLOG(D) + ONE / (F * 12) - ONE / (F**3 * 360)
     +          + ONE / (F**5 * 1260) - ONE / (F**7 * 1680)
     +          + ONE / (F**9 * 1188)
      END
