      PROGRAM ERWAY
C
C     PROGRAM ERWAY IS A DOUBLE PRECISION DRIVER PROGRAM, CALLING
C     A SELECTED SUBROUTINE TO COMPUTE FISHER'S EXACT PROBABILITY
C     FOR CROSS-CLASSIFICATION TABLES OF SIZES: 2 X 2, 3 X 2, 4 X 2,
C     3 X 3, 2 X 2 X 2, 5 X 2, AND 6 X 2.  THE CONSTANTS (INIT AND
C     EPSLN) ARE MACHINE-DEPENDENT VALUES TO COMPENSATE FOR ROUNDING
C     (EPSLN) AND TO PREVENT UNDER/OVER FLOW (INIT) IN ALL SUBROUTINES
C     (THESE VALUES MAY BE CHANGED TO ACCOMMODATE SPECIFIC MACHINES).
C
      INTEGER  CHOICE,ITAB,ITAIL,MX,E(6,3,2)
      REAL*8   P,P1,P2,P3,PX,INIT,EPSLN
      PARAMETER (INIT = 1.0D-150, EPSLN = 1.0D-12)
    1 WRITE  (*,2)
    2 FORMAT (/,T22,'FISHER EXACT PROBABILITY TESTS',
     &         //,T15,'(1)  FISHER EXACT TEST FOR 2 X 2 TABLES',
     &          /,T15,'(2)  FISHER EXACT TEST FOR 3 X 2 TABLES',
     &          /,T15,'(3)  FISHER EXACT TEST FOR 4 X 2 TABLES',
     &          /,T15,'(4)  FISHER EXACT TEST FOR 3 X 3 TABLES',
     &          /,T15,'(5)  FISHER EXACT TEST FOR 2 X 2 X 2 TABLES',
     &          /,T15,'(6)  FISHER EXACT TEST FOR 5 X 2 TABLES',
     &          /,T15,'(7)  FISHER EXACT TEST FOR 6 X 2 TABLES',
     &          /,T15,'(8)  EXIT FROM THE FISHER EXACT PROGRAM',
     &         //,T15,'     PLEASE CHOOSE A NUMBER FROM 1 TO 8')
      READ   (*,*) CHOICE
      IF(CHOICE.LT.1.OR.CHOICE.GT.8) STOP 'NUMBER NOT FROM 1 TO 8'
      GO TO (3,10,11,12,13,14,15,16) CHOICE
C
C     COMPUTE EXACT TEST FOR 2 BY 2 TABLES
C
    3 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 2 X 2 TABLES'
    4 FORMAT (/,T2,A,/)
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 2 X 2 TABLE'
    5 FORMAT (T2,A)
      READ   (*,*)  E(1,1,1),E(1,2,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 2 X 2 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1)
      CALL FE2X2 (E,INIT,EPSLN,P1,P2,P3,PX,ITAB,ITAIL,MX)
      WRITE  (*,6) '1ST ROW OF 2 X 2 TABLE: ',E(1,1,1),E(1,2,1)
    6 FORMAT (/,T5,A,3I9)
      WRITE  (*,7) '2ND ROW OF 2 X 2 TABLE: ',E(2,1,1),E(2,2,1)
    7 FORMAT (T5,A,3I9)
      IF(MX.EQ.1) THEN
         WRITE (*,5) 'THE ONE-SIDED P-VALUE IS LEFT TAILED'
      ELSE
         WRITE (*,5) 'THE ONE-SIDED P-VALUE IS RIGHT TAILED'
      END IF
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
    8 FORMAT (T5,A,I10)
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
    9 FORMAT (T5,A,G14.6E3)
      WRITE  (*,9) 'FISHER EXACT 2 X 2 TABLE ONE-SIDED P-VALUE = ',P1
      WRITE  (*,9) '1 + P(#: R1-C1 CELL) - (ONE-SIDED P-VALUE) = ',P2
      WRITE  (*,9) 'FISHER EXACT 2 X 2 TABLE TWO-SIDED P-VALUE = ',P3
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 3 X 2 TABLES
C
   10 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 3 X 2 TABLES'
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 3 X 2 TABLE'
      READ   (*,*)  E(1,1,1),E(1,2,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 3 X 2 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1)
      WRITE  (*,5) 'ENTER THE 3RD ROW OF THE 3 X 2 TABLE'
      READ   (*,*)  E(3,1,1),E(3,2,1)
      CALL FE3X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) '1ST ROW OF 3 X 2 TABLE: ',E(1,1,1),E(1,2,1)
      WRITE  (*,7) '2ND ROW OF 3 X 2 TABLE: ',E(2,1,1),E(2,2,1)
      WRITE  (*,7) '3RD ROW OF 3 X 2 TABLE: ',E(3,1,1),E(3,2,1)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
      WRITE  (*,9) 'FISHER EXACT 3 X 2 PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 4 BY 2 TABLES
C
   11 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 4 X 2 TABLES'
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 4 X 2 TABLE'
      READ   (*,*)  E(1,1,1),E(1,2,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 4 X 2 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1)
      WRITE  (*,5) 'ENTER THE 3RD ROW OF THE 4 X 2 TABLE'
      READ   (*,*)  E(3,1,1),E(3,2,1)
      WRITE  (*,5) 'ENTER THE 4TH ROW OF THE 4 X 2 TABLE'
      READ   (*,*)  E(4,1,1),E(4,2,1)
      CALL FE4X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) '1ST ROW OF 4 X 2 TABLE: ',E(1,1,1),E(1,2,1)
      WRITE  (*,7) '2ND ROW OF 4 X 2 TABLE: ',E(2,1,1),E(2,2,1)
      WRITE  (*,7) '3RD ROW OF 4 X 2 TABLE: ',E(3,1,1),E(3,2,1)
      WRITE  (*,7) '4TH ROW OF 4 X 2 TABLE: ',E(4,1,1),E(4,2,1)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX

      WRITE  (*,9) 'FISHER EXACT 4 X 2 PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 3 BY 3 TABLES
C
   12 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 3 X 3 TABLES'
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 3 X 3 TABLE'
      READ   (*,*)  E(1,1,1),E(1,2,1),E(1,3,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 3 X 3 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1),E(2,3,1)
      WRITE  (*,5) 'ENTER THE 3RD ROW OF THE 3 X 3 TABLE'
      READ   (*,*)  E(3,1,1),E(3,2,1),E(3,3,1)
      CALL FE3X3 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) '1ST ROW OF 3 X 3 TABLE: ',E(1,1,1),E(1,2,1),E(1,3,1)
      WRITE  (*,7) '2ND ROW OF 3 X 3 TABLE: ',E(2,1,1),E(2,2,1),E(2,3,1)
      WRITE  (*,7) '3RD ROW OF 3 X 3 TABLE: ',E(3,1,1),E(3,2,1),E(3,3,1)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
      WRITE  (*,9) 'FISHER EXACT 3 X 3 PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 2 BY 2 BY 2 TABLES
C
   13 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 2 X 2 X 2 TABLES'
      WRITE  (*,5) 'ENTER E111 AND E211 OF THE 2 X 2 X 2 TABLE'
      READ   (*,*)  E(1,1,1),E(2,1,1)
      WRITE  (*,5) 'ENTER E121 AND E221 OF THE 2 X 2 X 2 TABLE'
      READ   (*,*)  E(1,2,1),E(2,2,1)
      WRITE  (*,5) 'ENTER E112 AND E212 OF THE 2 X 2 X 2 TABLE'
      READ   (*,*)  E(1,1,2),E(2,1,2)
      WRITE  (*,5) 'ENTER E122 AND E222 OF THE 2 X 2 X 2 TABLE'
      READ   (*,*)  E(1,2,2),E(2,2,2)
      CALL FE2X2X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) 'E111 & E211 OF 2 X 2 X 2 TABLE: ',E(1,1,1),E(2,1,1)
      WRITE  (*,7) 'E121 & E221 OF 2 X 2 X 2 TABLE: ',E(1,2,1),E(2,2,1)
      WRITE  (*,7) 'E112 & E212 OF 2 X 2 X 2 TABLE: ',E(1,1,2),E(2,1,2)
      WRITE  (*,7) 'E122 & E222 OF 2 X 2 X 2 TABLE: ',E(1,2,2),E(2,2,2)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
      WRITE  (*,9) 'FISHER EXACT 2 X 2 X 2 PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 5 BY 2 TABLES
C
   14 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 5 X 2 TABLES'
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 5 X 2 TABLE'
      READ   (*,*)  E(1,1,1),E(1,2,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 5 X 2 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1)
      WRITE  (*,5) 'ENTER THE 3RD ROW OF THE 5 X 2 TABLE'
      READ   (*,*)  E(3,1,1),E(3,2,1)
      WRITE  (*,5) 'ENTER THE 4TH ROW OF THE 5 X 2 TABLE'
      READ   (*,*)  E(4,1,1),E(4,2,1)
      WRITE  (*,5) 'ENTER THE 5TH ROW OF THE 5 X 2 TABLE'
      READ   (*,*)  E(5,1,1),E(5,2,1)
      CALL FE5X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) '1ST ROW OF 5 X 2 TABLE: ',E(1,1,1),E(1,2,1)
      WRITE  (*,7) '2ND ROW OF 5 X 2 TABLE: ',E(2,1,1),E(2,2,1)
      WRITE  (*,7) '3RD ROW OF 5 X 2 TABLE: ',E(3,1,1),E(3,2,1)
      WRITE  (*,7) '4TH ROW OF 5 X 2 TABLE: ',E(4,1,1),E(4,2,1)
      WRITE  (*,7) '5TH ROW OF 5 X 2 TABLE: ',E(5,1,1),E(5,2,1)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
      WRITE  (*,9) 'FISHER EXACT 5 X 2 PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT TEST FOR 6 BY 2 TABLES
C
   15 WRITE  (*,4) 'FISHER''S EXACT TEST FOR 6 X 2 TABLES'
      WRITE  (*,5) 'ENTER THE 1ST ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(1,1,1),E(1,2,1)
      WRITE  (*,5) 'ENTER THE 2ND ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(2,1,1),E(2,2,1)
      WRITE  (*,5) 'ENTER THE 3RD ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(3,1,1),E(3,2,1)
      WRITE  (*,5) 'ENTER THE 4TH ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(4,1,1),E(4,2,1)
      WRITE  (*,5) 'ENTER THE 5TH ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(5,1,1),E(5,2,1)
      WRITE  (*,5) 'ENTER THE 6TH ROW OF THE 6 X 2 TABLE'
      READ   (*,*)  E(6,1,1),E(6,2,1)
      CALL  FE6X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
      WRITE  (*,6) '1ST ROW OF 6 X 2 TABLE: ',E(1,1,1),E(1,2,1)
      WRITE  (*,7) '2ND ROW OF 6 X 2 TABLE: ',E(2,1,1),E(2,2,1)
      WRITE  (*,7) '3RD ROW OF 6 X 2 TABLE: ',E(3,1,1),E(3,2,1)
      WRITE  (*,7) '4TH ROW OF 6 X 2 TABLE: ',E(4,1,1),E(4,2,1)
      WRITE  (*,7) '5TH ROW OF 6 X 2 TABLE: ',E(5,1,1),E(5,2,1)
      WRITE  (*,7) '6TH ROW OF 6 X 2 TABLE: ',E(6,1,1),E(6,2,1)
      WRITE  (*,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE  (*,8) '# OF TABLES COMPRISING EXACT P-VALUE = ',ITAIL
      WRITE  (*,9) 'POINT PROBABILITY OF OBSERVED TABLE = ',PX
      WRITE  (*,9) 'FISHER EXACT 6 X 2 PROBABILITY VALUE = ',P
      GO TO 1
C
C     EXIT THE FISHER EXACT TEST PROGRAM
C
   16 WRITE  (*,17) 'FISHER EXACT PROGRAM IS TERMINATED'
   17 FORMAT (T20,A,/)
      END
C
C
C
      SUBROUTINE FE2X2 (E,INIT,EPSLN,P1,P2,P3,PX,ITAB,ITAIL,MX)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P1    -- OUTPUT, EXACT ONE-TAILED PROBABILITY VALUE
C        P2    -- OUTPUT, OTHER ONE-TAILED PROBABILITY VALUE
C        P3    -- OUTPUT, EXACT TWO-TAILED PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT TWO-SIDED P-VALUE
C     USAGE:
C        SUBROUTINE FE2X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 2 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,X,Z,C1,R1,LLZ,ULZ,ITAB,ITAIL,E(6,3,2)
      REAL*8   H,T,LL,P1,P2,P3,T1,T2,U1,U2,U3,XX,ZZ,CC1,RR1,
     &         PX,INIT,EPSLN,ZERO,ONE,W
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      U2 = INIT
      H  = INIT
      T  = INIT
      T1 = ZERO
      T2 = ZERO
      U1 = ZERO
      U3 = ZERO
      X  = E(1,1,1)
      R1 = E(1,1,1)+E(1,2,1)
      C1 = E(1,1,1)+E(2,1,1)
      N  = R1+E(2,1,1)+E(2,2,1)
      K  = R1+C1-N
      L  = 1-K
      LL  = ONE*L
      MX  = 0
      XX  = ONE*X
      RR1 = ONE*R1
      CC1 = ONE*C1
      LLZ = MAX0(0,K)
      ULZ = MIN0(R1,C1)
C
C     BEGIN FIRST RECURSION LOOP
C
      IF (X.EQ.LLZ) W = U2
      IF (X.GT.LLZ) THEN
         DO 1 Z = LLZ,X-1
            ZZ = ONE*Z
            U2 = U2*(RR1-ZZ)*(CC1-ZZ)/(ZZ+ONE)/(LL+ZZ)
            IF(Z.EQ.X-1) W = U2
   1     CONTINUE
         U1 = U2*XX*(LL+XX-ONE)/(RR1-XX+ONE)/(CC1-XX+ONE)
      END IF
      IF (X.LT.ULZ) U3 = U2*(RR1-XX)*(CC1-XX)/(XX+ONE)/(LL+XX)
      U2 = U2*(ONE+EPSLN)
      IF (U1.LE.U3) THEN
         T1 = H
         MX = 1
      END IF
      ITAB = 1
      ITAIL = 0
      IF (H.LE.U2) THEN
         ITAIL = 1
         T2 = H
      END IF
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 2 Z = LLZ,ULZ-1
         ZZ = ONE*Z
         H  = H*(RR1-ZZ)*(CC1-ZZ)/(ZZ+ONE)/(LL+ZZ)
         ITAB = ITAB+1
         T  = T+H
         IF (U1.LE.U3.AND.Z.LE.X-1.OR.U1.GT.U3.AND.Z.GE.X-1) T1 = T1+H
         IF (H.LE.U2) THEN
            ITAIL  = ITAIL+1
            T2 = T2+H
         END IF
    2 CONTINUE
C
C     CALCULATE ONE-TAILED AND TWO-TAILED PROBABILITY VALUES
C     FOR THE 2 BY 2 CROSS-CLASSIFICATION TABLE
C
      P1 = T1/T
      P2 = ONE+(W/T)-P1
      P3 = T2/T
      PX = U2/T
      RETURN
      END
C
C
C
      SUBROUTINE FE3X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT NON-DIRECTIONAL PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE3X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 3 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,Y,Z,C1,R1,R2,Z1,LLY,LLZ,ULY,ULZ,ITAB,ITAIL,E(6,3,2)
      REAL*8   P,H,H1,LL,YY,ZZ,CC1,RR1,RR2,SUM,TOT,ZZ1,PX,
     &         INIT,EPSLN,CRIT,ZERO,ONE
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      H  = INIT
      H1 = INIT
      R1 = E(1,1,1)+E(1,2,1)
      R2 = E(2,1,1)+E(2,2,1)
      C1 = E(1,1,1)+E(2,1,1)+E(3,1,1)
      N  = R1+R2+E(3,1,1)+E(3,2,1)
      K  = R1+R2+C1-N
      L  = 1-K
      LL  = ONE*L
      RR1 = ONE*R1
      RR2 = ONE*R2
      CC1 = ONE*C1
      LLY = MAX0(0,K-R2)
      ULY = MIN0(R1,C1)
      SUM = ZERO
      TOT = ZERO
C
C     BEGIN FIRST RECURSION LOOP
C
      DO 2 Y = LLY,E(1,1,1)
         LLZ = MAX0(0,K-Y)
         ULZ = MIN0(R2,C1-Y)
         Z1  = LLZ
         YY  = ONE*Y
         ZZ1 = ONE*Z1
         DO 1 Z = LLZ,ULZ
            IF (Y.EQ.E(1,1,1).AND.Z.EQ.E(2,1,1)) THEN
               CRIT = H*(ONE+EPSLN)
               GO TO 3
            END IF
            ZZ = ONE*Z
            H  = H*(RR2-ZZ)*(CC1-YY-ZZ)/(ZZ+ONE)/(LL+YY+ZZ)
    1    CONTINUE
         IF (Z1.EQ.0) THEN
            H1 = H1*(RR1-YY)*(CC1-YY)/(YY+ONE)/(LL+YY)
         ELSE
            H1 = H1*(RR1-YY)*ZZ1/(YY+ONE)/(RR2-ZZ1+ONE)
         END IF
         H = H1
    2 CONTINUE
    3 H  = INIT
      H1 = INIT
      ITAB = 0
      ITAIL = 0
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 5 Y = LLY,ULY
         LLZ = MAX0(0,K-Y)
         ULZ = MIN0(R2,C1-Y)
         Z1  = LLZ
         YY  = ONE*Y
         ZZ1 = ONE*Z1
         DO 4 Z = LLZ,ULZ
            ZZ  = ONE*Z
            ITAB =  ITAB+1
            TOT = TOT+H
            IF (H.LE.CRIT) THEN
               ITAIL =  ITAIL+1
               SUM = SUM+H
            END IF
            H = H*(RR2-ZZ)*(CC1-YY-ZZ)/(ZZ+ONE)/(LL+YY+ZZ)
    4    CONTINUE
         IF (Z1.EQ.0) THEN
            H1 = H1*(RR1-YY)*(CC1-YY)/(YY+ONE)/(LL+YY)
         ELSE
            H1 = H1*(RR1-YY)*ZZ1/(YY+ONE)/(RR2-ZZ1+ONE)
         END IF
         H = H1
    5 CONTINUE
C
C     CALCULATE THE PROBABILITY VALUE FOR THE 3 BY 2
C     CROSS-CLASSIFICATION TABLE
C
      P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
C
C
C
      SUBROUTINE FE4X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT NON-DIRECTIONAL PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE4X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 4 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,X,Y,Z,C1,R1,R2,R3,Y2,Z1,Z2,LLX,LLY,LLZ,ULX,
     &         ULY,ULZ,ITAB,ITAIL,E(6,3,2)
      REAL*8   P,H,H1,H2,LL,XX,YY,ZZ,CC1,RR1,RR2,RR3,SUM,PX,
     &         TOT,YY2,ZZ1,ZZ2,INIT,EPSLN,CRIT,ZERO,ONE
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      H  = INIT
      H1 = INIT
      H2 = INIT
      R1 = E(1,1,1)+E(1,2,1)
      R2 = E(2,1,1)+E(2,2,1)
      R3 = E(3,1,1)+E(3,2,1)
      C1 = E(1,1,1)+E(2,1,1)+E(3,1,1)+E(4,1,1)
      N  = R1+R2+R3+E(4,1,1)+E(4,2,1)
      K  = R1+R2+R3+C1-N
      L  = 1-K
      LL  = ONE*L
      RR1 = ONE*R1
      RR2 = ONE*R2
      RR3 = ONE*R3
      CC1 = ONE*C1
      LLX = MAX0(0,K-R2-R3)
      ULX = MIN0(R1,C1)
      TOT = ZERO
      SUM = ZERO
C
C     BEGIN FIRST RECURSION LOOP
C
      DO 3 X = LLX,E(1,1,1)
         LLY = MAX0(0,K-R3-X)
         ULY = MIN0(R2,C1-X)
         Y2  = LLY
         Z2  = MAX0(0,K-X-Y2)
         XX  = ONE*X
         YY2 = ONE*Y2
         ZZ2 = ONE*Z2
         IF (X.EQ.E(1,1,1)) ULY = E(2,1,1)
         DO 2 Y = LLY,ULY
            LLZ = MAX0(0,K-X-Y)
            ULZ = MIN0(R3,C1-X-Y)
            Z1  = LLZ
            YY  = ONE*Y
            ZZ1 = ONE*Z1
            DO 1 Z = LLZ,ULZ
               IF (X.EQ.E(1,1,1).AND.Y.EQ.E(2,1,1).AND.Z.EQ
     &            .E(3,1,1)) THEN
                  CRIT = H*(ONE+EPSLN)
                  GO TO 4
               END IF
               ZZ = ONE*Z
               H  = H*(RR3-ZZ)*(CC1-XX-YY-ZZ)/(ZZ+ONE)/(LL+XX+YY+ZZ)
    1       CONTINUE
            IF (Z1.EQ.0) THEN
               H1 = H1*(RR2-YY)*(CC1-XX-YY)/(YY+ONE)/(LL+XX+YY)
            ELSE
               H1 = H1*(RR2-YY)*ZZ1/(YY+ONE)/(RR3-ZZ1+ONE)
            END IF
            H = H1
    2    CONTINUE
         IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
            H2 = H2*(RR1-XX)*(CC1-XX)/(XX+ONE)/(LL+XX)
         ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
            H2 = H2*(RR1-XX)*ZZ2/(XX+ONE)/(RR3-ZZ2+ONE)
         ELSE
            H2 = H2*(RR1-XX)*YY2/(XX+ONE)/(RR2-YY2+ONE)
         END IF
         H  = H2
         H1 = H2
    3 CONTINUE
    4 H  = INIT
      H1 = INIT
      H2 = INIT
      ITAB = 0
      ITAIL = 0
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 7 X = LLX,ULX
         LLY = MAX0(0,K-R3-X)
         ULY = MIN0(R2,C1-X)
         Y2  = LLY
         Z2  = MAX0(0,K-X-Y2)
         XX  = ONE*X
         YY2 = ONE*Y2
         ZZ2 = ONE*Z2
         DO 6 Y = LLY,ULY
            LLZ = MAX0(0,K-X-Y)
            ULZ = MIN0(R3,C1-X-Y)
            Z1  = LLZ
            YY  = ONE*Y
            ZZ1 = ONE*Z1
            DO 5 Z = LLZ,ULZ
               ZZ  = ONE*Z
               ITAB = ITAB+1
               TOT = TOT+H
               IF (H.LE.CRIT) THEN
                  ITAIL = ITAIL+1
                  SUM = SUM+H
               END IF
               H = H*(RR3-ZZ)*(CC1-XX-YY-ZZ)/(ZZ+ONE)/(LL+XX+YY+ZZ)
    5       CONTINUE
            IF (Z1.EQ.0) THEN
               H1 = H1*(RR2-YY)*(CC1-XX-YY)/(YY+ONE)/(LL+XX+YY)
            ELSE
               H1 = H1*(RR2-YY)*ZZ1/(YY+ONE)/(RR3-ZZ1+ONE)
            END IF
            H = H1
    6    CONTINUE
         IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
            H2 = H2*(RR1-XX)*(CC1-XX)/(XX+ONE)/(LL+XX)
         ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
            H2 = H2*(RR1-XX)*ZZ2/(XX+ONE)/(RR3-ZZ2+ONE)
         ELSE
            H2 = H2*(RR1-XX)*YY2/(XX+ONE)/(RR2-YY2+ONE)
         END IF
         H  = H2
         H1 = H2
    7 CONTINUE
C
C     CALCULATE THE PROBABILITY VALUE FOR THE 4 BY 2
C     CROSS-CLASSIFICATION TABLE
C
      P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
C
C
C
      SUBROUTINE FE3X3 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT NON-DIRECTIONAL PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE3X3 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 3 BY 3 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,W,X,Y,Z,C1,C2,R1,R2,X3,Y2,Y3,Z1,Z2,Z3,LLW,LLX,
     &         LLY,LLZ,ULW,ULX,ULY,ULZ,ITAB,ITAIL,E(6,3,2)
      REAL*8   P,H,H1,H2,H3,KK,LL,WW,XX,YY,ZZ,CC1,CC2,RR1,RR2,SUM,TOT,
     &         XX3,YY2,YY3,ZZ1,ZZ2,ZZ3,INIT,PX,EPSLN,CRIT,ZERO,ONE
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      R1 = E(1,1,1)+E(1,2,1)+E(1,3,1)
      R2 = E(2,1,1)+E(2,2,1)+E(2,3,1)
      C1 = E(1,1,1)+E(2,1,1)+E(3,1,1)
      C2 = E(1,2,1)+E(2,2,1)+E(3,2,1)
      N  = R1+R2+E(3,1,1)+E(3,2,1)+E(3,3,1)
      K  = R1+R2+C1+C2-N
      L  = 1-K
      KK  = ONE*K
      LL  = ONE*L
      RR1 = ONE*R1
      RR2 = ONE*R2
      CC1 = ONE*C1
      CC2 = ONE*C2
      LLW = MAX0(0,K-R2-C2)
      ULW = MIN0(R1,C1)
      TOT = ZERO
      SUM = ZERO
C
C     BEGIN FIRST RECURSION LOOP
C
      DO 4 W = LLW,E(1,1,1)
         LLX = MAX0(0,K-R2-W)
         ULX = MIN0(R1-W,C2)
         X3  = LLX
         Y3  = MAX0(0,K-C2-W)
         Z3  = MAX0(0,K-W-X3-Y3)
         WW  = ONE*W
         XX3 = ONE*X3
         YY3 = ONE*Y3
         ZZ3 = ONE*Z3
         IF (W.EQ.E(1,1,1)) ULX = E(1,2,1)
         DO 3 X = LLX,ULX
            LLY = MAX0(0,K-C2-W)
            ULY = MIN0(R2,C1-W)
            Y2  = LLY
            Z2  = MAX0(0,K-W-X-Y2)
            XX  = ONE*X
            YY2 = ONE*Y2
            ZZ2 = ONE*Z2
            IF (W.EQ.E(1,1,1).AND.X.EQ.E(1,2,1)) ULY  = E(2,1,1)
            DO 2 Y = LLY,ULY
               LLZ = MAX0(0,K-W-X-Y)
               ULZ = MIN0(R2-Y,C2-X)
               Z1  = LLZ
               YY  = ONE*Y
               ZZ1 = ONE*Z1
               DO 1 Z = LLZ,ULZ
                  IF (W.EQ.E(1,1,1).AND.X.EQ.E(1,2,1).AND.Y.EQ.E(2,1,1)
     &               .AND.Z.EQ.E(2,2,1)) THEN
                     CRIT = H*(ONE+EPSLN)
                     GO TO 5
                  END IF
                  ZZ = ONE*Z
                  H  = H*(RR2-YY-ZZ)*(CC2-XX-ZZ)/(ZZ+ONE)/
     &                 (LL+WW+XX+YY+ZZ)
    1          CONTINUE
               IF (Z1.EQ.0) THEN
                  H1 = H1*(RR2-YY)*(CC1-WW-YY)/(YY+ONE)/(LL+WW+XX+YY)
               ELSE
                  H1 = H1*(CC1-WW-YY)*ZZ1/(YY+ONE)/(CC2-XX-ZZ1+ONE)
               END IF
               H = H1
    2       CONTINUE
            IF (Z2.EQ.0) THEN
               H2 = H2*(RR1-WW-XX)*(CC2-XX)/(XX+ONE)/(LL+WW+XX+YY2)
            ELSE
               H2 = H2*(RR1-WW-XX)*ZZ2/(XX+ONE)/(RR2-YY2-ZZ2+ONE)
            END IF
            H  = H2
            H1 = H2
    3    CONTINUE
         IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)/(WW+ONE)/(LL+WW)
         ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)*ZZ3/(WW+ONE)/(RR2-ZZ3+ONE)/
     &           (CC2-ZZ3+ONE)
         ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
            H3 = H3*(RR1-WW)*YY3/(WW+ONE)/(RR2-YY3-ZZ3+ONE)
         ELSE IF (X3.GT.0.AND.Y3.EQ.0) THEN
            H3 = H3*(CC1-WW)*XX3/(WW+ONE)/(CC2-XX3-ZZ3+ONE)
         ELSE IF (X3.GT.0.AND.Y3.GT.0.AND.K.LT.W+X3+Y3) THEN
            H3 = H3*(WW+XX3+YY3-KK)*XX3*YY3/(WW+ONE)/(RR2-YY3+ONE)/
     &           (CC2-XX3+ONE)
         ELSE
            H3 = H3*XX3*YY3/(WW+ONE)/(ZZ3+ONE)
         END IF
         H  = H3
         H1 = H3
         H2 = H3
    4 CONTINUE
    5 H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      ITAB = 0
      ITAIL = 0
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 9 W = LLW,ULW
         LLX = MAX0(0,K-R2-W)
         ULX = MIN0(R1-W,C2)
         X3  = LLX
         Y3  = MAX0(0,K-C2-W)
         Z3  = MAX0(0,K-W-X3-Y3)
         WW  = ONE*W
         XX3 = ONE*X3
         YY3 = ONE*Y3
         ZZ3 = ONE*Z3
         DO 8 X = LLX,ULX
            LLY = MAX0(0,K-C2-W)
            ULY = MIN0(R2,C1-W)
            Y2  = LLY
            Z2  = MAX0(0,K-W-X-Y2)
            XX  = ONE*X
            YY2 = ONE*Y2
            ZZ2 = ONE*Z2
            DO 7 Y = LLY,ULY
               LLZ = MAX0(0,K-W-X-Y)
               ULZ = MIN0(R2-Y,C2-X)
               Z1  = LLZ
               YY  = ONE*Y
               ZZ1 = ONE*Z1
               DO 6 Z = LLZ,ULZ
                  ITAB =  ITAB+1
                  TOT = TOT+H
                  IF (H.LE.CRIT) THEN
                     ITAIL = ITAIL+1
                     SUM = SUM+H
                  END IF
                  ZZ = ONE*Z
                  H  = H*(RR2-YY-ZZ)*(CC2-XX-ZZ)/(ZZ+ONE)/
     &                 (LL+WW+XX+YY+ZZ)
    6          CONTINUE
               IF (Z1.EQ.0) THEN
                  H1 = H1*(RR2-YY)*(CC1-WW-YY)/(YY+ONE)/(LL+WW+XX+YY)
               ELSE
                  H1 = H1*(CC1-WW-YY)*ZZ1/(YY+ONE)/(CC2-XX-ZZ1+ONE)
               END IF
               H = H1
    7       CONTINUE
            IF (Z2.EQ.0) THEN
               H2 = H2*(RR1-WW-XX)*(CC2-XX)/(XX+ONE)/(LL+WW+XX+YY2)
            ELSE
               H2 = H2*(RR1-WW-XX)*ZZ2/(XX+ONE)/(RR2-YY2-ZZ2+ONE)
            END IF
            H  = H2
            H1 = H2
    8    CONTINUE
         IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)/(WW+ONE)/(LL+WW)
         ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)*ZZ3/(WW+ONE)/(RR2-ZZ3+ONE)/
     &           (CC2-ZZ3+ONE)
         ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
            H3 = H3*(RR1-WW)*YY3/(WW+ONE)/(RR2-YY3-ZZ3+ONE)
         ELSE IF (X3.GT.0.AND.Y3.EQ.0) THEN
            H3 = H3*(CC1-WW)*XX3/(WW+ONE)/(CC2-XX3-ZZ3+ONE)
         ELSE IF (X3.GT.0.AND.Y3.GT.0.AND.K.LT.W+X3+Y3) THEN
            H3 = H3*(WW+XX3+YY3-KK)*XX3*YY3/(WW+ONE)/(RR2-YY3+ONE)/
     &           (CC2-XX3+ONE)
         ELSE
            H3 = H3*XX3*YY3/(WW+ONE)/(ZZ3+ONE)
         END IF
         H  = H3
         H1 = H3
         H2 = H3
    9 CONTINUE
C
C     CALCULATE THE PROBABILITY VALUE FOR THE 3 BY 3
C     CROSS-CLASSIFICATION TABLE
C
      P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
C
C
C
      SUBROUTINE FE2X2X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING P-VALUE
C     USAGE:
C        SUBROUTINE FE2X2X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 2 BY 2 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  ITAB,ITAIL,IA,IB,IC,IT,MA,MB,MC,MABC,LZ,MW,MX,MY,MZ,NT,
     &         IW,IX,IY,IZ,I,J,K,IZ1,IZ1A,IZ2,IZ2A,IZ3,IZ3A,DZ1,DZ2,DZ3,
     &         E(6,3,2),N(2,2,2)
      REAL*8   INIT,EPSLN,P,PX,ONE,ZERO,A,B,C,ABC,H,H1,H2,H3,CRIT,SUM,
     &         TOT,W,X,Y,Z,Z1,Z2,Z3
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C  INITIALIZE VARIABLES
C
      IA = E(1,1,1)+E(1,1,2)+E(1,2,1)+E(1,2,2)
      IB = E(1,1,1)+E(1,1,2)+E(2,1,1)+E(2,1,2)
      IC = E(1,1,1)+E(1,2,1)+E(2,1,1)+E(2,2,1)
      IT = IA+E(2,1,1)+E(2,1,2)+E(2,2,1)+E(2,2,2)
      NT = IDINT(0.1D0+ONE*IT/2)
      DO 30 I = 1,2
        DO 20 J = 1,2
          DO 10 K = 1,2
            IF(IA.GT.NT.AND.IB.GT.NT.AND.IC.GT.NT) THEN
              N(I,J,K) = E(3-I,3-J,3-K)
            ELSE IF(IA.GT.NT.AND.IB.GT.NT) THEN
              N(I,J,K) = E(3-I,3-J,K)
            ELSE IF(IA.GT.NT.AND.IC.GT.NT) THEN
              N(I,J,K) = E(3-I,J,3-K)
            ELSE IF(IB.GT.NT.AND.IC.GT.NT) THEN
              N(I,J,K) = E(I,3-J,3-K)
            ELSE IF(IA.GT.NT) THEN
              N(I,J,K) = E(3-I,J,K)
            ELSE IF(IB.GT.NT) THEN
              N(I,J,K) = E(I,3-J,K)
            ELSE IF(IC.GT.NT) THEN
              N(I,J,K) = E(I,J,3-K)
            ELSE
              N(I,J,K) = E(I,J,K)
            END IF
   10     CONTINUE
   20   CONTINUE
   30 CONTINUE
      MA = MIN0(IA,IT-IA)
      MB = MIN0(IB,IT-IB)
      MC = MIN0(IC,IT-IC)
      MABC = MA+MB+MC-IT
      A = ONE*MA
      B = ONE*MB
      C = ONE*MC
      ABC = ONE*MABC
      MW = MIN0(MA,MB,MC)
      IZ3A = MAX0(0,MABC)
      H = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
C
C  BEGIN FIRST RECURSION LOOP
C
      DO 70 IW = 0,MW
        W = ONE*IW
        MX = MIN0(MA-IW,MB-IW)
        IZ3 = IZ3A
        IZ2A = IZ3
        IZ3A = MAX0(0,MABC-IW*2-2)
        DZ3 = IZ3-IZ3A
        Z3 = ONE*IZ3
        DO 60 IX = 0,MX
          X = ONE*IX
          MY = MIN0(MA-IW-IX,MC-IW)
          IZ2 = IZ2A
          IZ1A = IZ2
          IZ2A = MAX0(0,MABC-IW*2-IX-1)
          DZ2 = IZ2-IZ2A
          Z2 = ONE*IZ2
          DO 50 IY = 0,MY
            Y = ONE*IY
            LZ = MAX0(0,MABC-IW*2-IX-IY)
            MZ = MIN0(MB-IW-IX,MC-IW-IY)
            IZ1 = IZ1A
            IZ1A = MAX0(0,MABC-IW*2-IX-IY-1)
            DZ1 = IZ1-IZ1A
            Z1 = ONE*IZ1
            DO 40 IZ = LZ,MZ
              Z = ONE*IZ
              IF(IW.EQ.N(1,1,1).AND.IX.EQ.N(1,1,2).AND.IY.EQ.N(1,2,1)
     &             .AND.IZ.EQ.N(2,1,1)) THEN
                CRIT = H*(ONE+EPSLN)
                GO TO 80
              END IF
              H = H*(B-W-X-Z)*(C-W-Y-Z)/(Z+1)/(W*2+X+Y+Z+1-ABC)
   40       CONTINUE
            IF(DZ1.EQ.1) THEN
              H1 = H1*Z1*(A-W-X-Y)/(Y+1)/(B-W-X-Z1+1)
            ELSE
              H1 = H1*(A-W-X-Y)*(C-W-Y)/(Y+1)/(W*2+X+Y+1-ABC)
            END IF
            H = H1
   50     CONTINUE
          IF(DZ2.EQ.1) THEN
            H2 = H2*Z2*(A-W-X)/(X+1)/(C-W-Z2+1)
          ELSE
            H2 = H2*(A-W-X)*(B-W-X)/(X+1)/(W*2+X+1-ABC)
          END IF
          H = H2
          H1 = H2
   60   CONTINUE
        IF(DZ3.EQ.2) THEN
          H3 = H3*Z3*(Z3-1)*(A-W)/(W+1)/(B-W-Z3+1)/(C-W-Z3+1)
        ELSE IF(DZ3.EQ.1) THEN
          H3 = H3*(A-W)/(W+1)/(W*2+2-ABC)
        ELSE
          H3 = H3*(A-W)*(B-W)*(C-W)/(W+1)/(W*2+2-ABC)/(W*2+1-ABC)
        END IF
        H = H3
        H1 = H3
        H2 = H3
   70 CONTINUE
   80 H = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      IZ3A = MAX0(0,MABC)
      ITAB = 0
      ITAIL = 0
      TOT = ZERO
      SUM  = ZERO
C
C  BEGIN SECOND RECURSION LOOP
C
      DO 120 IW = 0,MW
        W = ONE*IW
        MX = MIN0(MA-IW,MB-IW)
        IZ3 = IZ3A
        IZ2A = IZ3
        IZ3A = MAX0(0,MABC-IW*2-2)
        DZ3 = IZ3-IZ3A
        Z3 = ONE*IZ3
        DO 110 IX = 0,MX
          X = ONE*IX
          MY = MIN0(MA-IW-IX,MC-IW)
          IZ2 = IZ2A
          IZ1A = IZ2
          IZ2A = MAX0(0,MABC-IW*2-IX-1)
          DZ2 = IZ2-IZ2A
          Z2 = ONE*IZ2
          DO 100 IY = 0,MY
            Y = ONE*IY
            LZ = MAX0(0,MABC-IW*2-IX-IY)
            MZ = MIN0(MB-IW-IX,MC-IW-IY)
            IZ1 = IZ1A
            IZ1A = MAX0(0,MABC-IW*2-IX-IY-1)
            DZ1 = IZ1-IZ1A
            Z1 = ONE*IZ1
            DO 90 IZ = LZ,MZ
              Z = ONE*IZ
              ITAB = ITAB+1
              TOT = TOT+H
              IF(H.LE.CRIT) THEN
                ITAIL = ITAIL+1
                SUM = SUM+H
              END IF
              H = H*(B-W-X-Z)*(C-W-Y-Z)/(Z+1)/(W*2+X+Y+Z+1-ABC)
   90       CONTINUE
            IF(DZ1.EQ.1) THEN
              H1 = H1*Z1*(A-W-X-Y)/(Y+1)/(B-W-X-Z1+1)
            ELSE
              H1 = H1*(A-W-X-Y)*(C-W-Y)/(Y+1)/(W*2+X+Y+1-ABC)
            END IF
            H = H1
  100     CONTINUE
          IF(DZ2.EQ.1) THEN
            H2 = H2*Z2*(A-W-X)/(X+1)/(C-W-Z2+1)
          ELSE
            H2 = H2*(A-W-X)*(B-W-X)/(X+1)/(W*2+X+1-ABC)
          END IF
          H = H2
          H1 = H2
  110   CONTINUE
        IF(DZ3.EQ.2) THEN
          H3 = H3*Z3*(Z3-1)*(A-W)/(W+1)/(B-W-Z3+1)/(C-W-Z3+1)
        ELSE IF(DZ3.EQ.1) THEN
          H3 = H3*(A-W)/(W+1)/(W*2+2-ABC)
        ELSE
          H3 = H3*(A-W)*(B-W)*(C-W)/(W+1)/(W*2+2-ABC)/(W*2+1-ABC)
        END IF
        H = H3
        H1 = H3
        H2 = H3
  120 CONTINUE
C
C     CALCULATE THE EXACT P-VALUE AND POINT PROBABILITY VALUE
C     OF THE OBSERVED 2 BY 2 BY 2 CROSS-CLASSIFICATION TABLE
C
  150 P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
C
C
C
      SUBROUTINE FE5X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT NON-DIRECTIONAL PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE5X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 5 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,W,X,Y,Z,C1,R1,R2,R3,R4,X3,Y2,Y3,Z1,Z2,Z3,LLW,LLX,
     &         LLY,LLZ,ULW,ULX,ULY,ULZ,ITAB,ITAIL,E(6,3,2)
      REAL*8   P,H,H1,H2,H3,LL,WW,XX,YY,ZZ,CC1,RR1,RR2,RR3,RR4,SUM,TOT,
     &         XX3,YY2,YY3,ZZ1,ZZ2,ZZ3,INIT,PX,EPSLN,CRIT,ZERO,ONE
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      R1 = E(1,1,1)+E(1,2,1)
      R2 = E(2,1,1)+E(2,2,1)
      R3 = E(3,1,1)+E(3,2,1)
      R4 = E(4,1,1)+E(4,2,1)
      C1 = E(1,1,1)+E(2,1,1)+E(3,1,1)+E(4,1,1)+E(5,1,1)
      N  = R1+R2+R3+R4+E(5,1,1)+E(5,2,1)
      K  = R1+R2+R3+R4+C1-N
      L  = 1-K
      LL  = ONE*L
      RR1 = ONE*R1
      RR2 = ONE*R2
      RR3 = ONE*R3
      RR4 = ONE*R4
      CC1 = ONE*C1
      LLW = MAX0(0,K-R2-R3-R4)
      ULW = MIN0(R1,C1)
      TOT = ZERO
      SUM = ZERO
C
C     BEGIN FIRST RECURSION LOOP
C
      DO 4 W = LLW,E(1,1,1)
         LLX = MAX0(0,K-R3-R4-W)
         ULX = MIN0(R2,C1-W)
         X3  = LLX
         Y3  = MAX0(0,K-R4-W-X3)
         Z3  = MAX0(0,K-W-X3-Y3)
         WW  = ONE*W
         XX3 = ONE*X3
         YY3 = ONE*Y3
         ZZ3 = ONE*Z3
         IF (W.EQ.E(1,1,1)) ULX = E(2,1,1)
         DO 3 X = LLX,ULX
            LLY = MAX0(0,K-R4-W-X)
            ULY = MIN0(R3,C1-W-X)
            Y2  = LLY
            Z2  = MAX0(0,K-W-X-Y2)
            XX  = ONE*X
            YY2 = ONE*Y2
            ZZ2 = ONE*Z2
            IF (W.EQ.E(1,1,1).AND.X.EQ.E(2,1,1)) ULY = E(3,1,1)
            DO 2 Y = LLY,ULY
               LLZ = MAX0(0,K-W-X-Y)
               ULZ = MIN0(R4,C1-W-X-Y)
               Z1  = LLZ
               YY  = ONE*Y
               ZZ1 = ONE*Z1
               DO 1 Z = LLZ,ULZ
                  IF (W.EQ.E(1,1,1).AND.X.EQ.E(2,1,1).AND.Y.EQ.E(3,1,1)
     &               .AND.Z.EQ.E(4,1,1)) THEN
                     CRIT = H*(ONE+EPSLN)
                     GO TO 5
                  END IF
                  ZZ = ONE*Z
                  H  = H*(RR4-ZZ)*(CC1-WW-XX-YY-ZZ)/(ZZ+ONE)/
     &                 (LL+WW+XX+YY+ZZ)
    1          CONTINUE
               IF (Z1.EQ.0) THEN
                  H1 = H1*(RR3-YY)*(CC1-WW-XX-YY)/(YY+ONE)/
     &                 (LL+WW+XX+YY)
               ELSE
                  H1 = H1*(RR3-YY)*ZZ1/(YY+ONE)/(RR4-ZZ1+ONE)
               END IF
               H = H1
    2       CONTINUE
            IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
               H2 = H2*(RR2-XX)*(CC1-WW-XX)/(XX+ONE)/(LL+WW+XX)
            ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
               H2 = H2*(RR2-XX)*ZZ2/(XX+ONE)/(RR4-ZZ2+ONE)
            ELSE
               H2 = H2*(RR2-XX)*YY2/(XX+ONE)/(RR3-YY2+ONE)
            END IF
            H  = H2
            H1 = H2
    3    CONTINUE
         IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)/(WW+ONE)/(LL+WW)
         ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
            H3 = H3*(RR1-WW)*ZZ3/(WW+ONE)/(RR4-ZZ3+ONE)
         ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
            H3 = H3*(RR1-WW)*YY3/(WW+ONE)/(RR3-YY3+ONE)
         ELSE
            H3 = H3*(RR1-WW)*XX3/(WW+ONE)/(RR2-XX3+ONE)
         END IF
         H  = H3
         H1 = H3
         H2 = H3
    4 CONTINUE
    5 H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      ITAB = 0
      ITAIL = 0
C
C          BEGIN SECOND RECURSION LOOP
C
      DO 9 W = LLW,ULW
         LLX = MAX0(0,K-R3-R4-W)
         ULX = MIN0(R2,C1-W)
         X3  = LLX
         Y3  = MAX0(0,K-R4-W-X3)
         Z3  = MAX0(0,K-W-X3-Y3)
         WW  = ONE*W
         XX3 = ONE*X3
         YY3 = ONE*Y3
         ZZ3 = ONE*Z3
         DO 8 X = LLX,ULX
            LLY = MAX0(0,K-R4-W-X)
            ULY = MIN0(R3,C1-W-X)
            Y2  = LLY
            Z2  = MAX0(0,K-W-X-Y2)
            XX  = ONE*X
            YY2 = ONE*Y2
            ZZ2 = ONE*Z2
            DO 7 Y = LLY,ULY
               LLZ = MAX0(0,K-W-X-Y)
               ULZ = MIN0(R4,C1-W-X-Y)
               Z1  = LLZ
               YY  = ONE*Y
               ZZ1 = ONE*Z1
               DO 6 Z = LLZ,ULZ
                  ITAB = ITAB+1
                  TOT = TOT+H
                  IF (H.LE.CRIT) THEN
                     ITAIL = ITAIL+1
                     SUM = SUM+H
                  END IF
                  ZZ = ONE*Z
                  H  = H*(RR4-ZZ)*(CC1-WW-XX-YY-ZZ)/(ZZ+ONE)/
     &                (LL+WW+XX+YY+ZZ)
    6          CONTINUE
               IF (Z1.EQ.0) THEN
                  H1 = H1*(RR3-YY)*(CC1-WW-XX-YY)/(YY+ONE)/(LL+WW+XX+YY)
               ELSE
                  H1 = H1*(RR3-YY)*ZZ1/(YY+ONE)/(RR4-ZZ1+ONE)
               END IF
               H = H1
    7       CONTINUE
            IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
               H2 = H2*(RR2-XX)*(CC1-WW-XX)/(XX+ONE)/(LL+WW+XX)
            ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
               H2 = H2*(RR2-XX)*ZZ2/(XX+ONE)/(RR4-ZZ2+ONE)
            ELSE
               H2 = H2*(RR2-XX)*YY2/(XX+ONE)/(RR3-YY2+ONE)
            END IF
            H  = H2
            H1 = H2
    8    CONTINUE
         IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
            H3 = H3*(RR1-WW)*(CC1-WW)/(WW+ONE)/(LL+WW)
         ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
            H3 = H3*(RR1-WW)*ZZ3/(WW+ONE)/(RR4-ZZ3+ONE)
         ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
            H3 = H3*(RR1-WW)*YY3/(WW+ONE)/(RR3-YY3+ONE)
         ELSE
            H3 = H3*(RR1-WW)*XX3/(WW+ONE)/(RR2-XX3+ONE)
         END IF
         H  = H3
         H1 = H3
         H2 = H3
    9 CONTINUE
C
C     CALCULATE THE PROBABILITY VALUE FOR THE 5 BY 2
C     CROSS-CLASSIFICATION TABLE
C
      P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
C
C
C
      SUBROUTINE FE6X2 (E,INIT,EPSLN,P,PX,ITAB,ITAIL)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        P     -- OUTPUT, EXACT NON-DIRECTIONAL PROBABILITY VALUE
C        PX    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE'
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TABLES
C        ITAIL -- OUTPUT, # OF TABLES COMPRISING EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE6X2 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 6 BY 2 CROSS-CLASSIFICATION TABLES
C
      INTEGER  K,L,N,V,W,X,Y,Z,C1,R1,R2,R3,R4,R5,W4,X3,X4,Y2,Y3,Y4,Z1,
     &         Z2,Z3,Z4,LLV,LLW,LLX,LLY,LLZ,ULV,ULW,ULX,ULY,ULZ,
     &         ITAB,ITAIL,E(6,3,2)
      REAL*8   P,H,H1,H2,H3,H4,LL,VV,WW,XX,YY,ZZ,CC1,RR1,RR2,RR3,RR4,
     &         RR5,SUM,TOT,WW4,XX3,XX4,YY2,YY3,YY4,ZZ1,ZZ2,ZZ3,ZZ4,
     &         PX,INIT,EPSLN,CRIT,ZERO,ONE
      DATA  ZERO,ONE /0.0D0,1.0D0/
C
C     INITIALIZE VARIABLES
C
      H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      H4 = INIT
      R1 = E(1,1,1)+E(1,2,1)
      R2 = E(2,1,1)+E(2,2,1)
      R3 = E(3,1,1)+E(3,2,1)
      R4 = E(4,1,1)+E(4,2,1)
      R5 = E(5,1,1)+E(5,2,1)
      C1 = E(1,1,1)+E(2,1,1)+E(3,1,1)+E(4,1,1)+E(5,1,1)+E(6,1,1)
      N  = R1+R2+R3+R4+R5+E(6,1,1)+E(6,2,1)
      K  = R1+R2+R3+R4+R5+C1-N
      L  = 1-K
      LL  = ONE*L
      RR1 = ONE*R1
      RR2 = ONE*R2
      RR3 = ONE*R3
      RR4 = ONE*R4
      RR5 = ONE*R5
      CC1 = ONE*C1
      LLV = MAX0(0,K-R2-R3-R4-R5)
      ULV = MIN0(R1,C1)
      TOT = ZERO
      SUM = ZERO
C
C     BEGIN FIRST RECURSION LOOP
C
      DO 5 V = LLV,E(1,1,1)
         LLW = MAX0(0,K-R3-R4-R5-V)
         ULW = MIN0(R2,C1-V)
         W4  = LLW
         X4  = MAX0(0,K-R4-R5-V-W4)
         Y4  = MAX0(0,K-R5-V-W4-X4)
         Z4  = MAX0(0,K-V-W4-X4-Y4)
         VV  = ONE*V
         WW4 = ONE*W4
         XX4 = ONE*X4
         YY4 = ONE*Y4
         ZZ4 = ONE*Z4
         IF (V.EQ.E(1,1,1)) ULW = E(2,1,1)
         DO 4 W = LLW,ULW
            LLX = MAX0(0,K-R4-R5-V-W)
            ULX = MIN0(R3,C1-V-W)
            X3  = LLX
            Y3  = MAX0(0,K-R5-V-W-X3)
            Z3  = MAX0(0,K-V-W-X3-Y3)
            WW  = ONE*W
            XX3 = ONE*X3
            YY3 = ONE*Y3
            ZZ3 = ONE*Z3
            IF (V.EQ.E(1,1,1).AND.W.EQ.E(2,1,1)) ULX = E(3,1,1)
            DO 3 X = LLX,ULX
               LLY = MAX0(0,K-R5-V-W-X)
               ULY = MIN0(R4,C1-V-W-X)
               Y2  = LLY
               Z2  = MAX0(0,K-V-W-X-Y2)
               XX  = ONE*X
               YY2 = ONE*Y2
               ZZ2 = ONE*Z2
               IF (V.EQ.E(1,1,1).AND.W.EQ.E(2,1,1).AND.X.EQ.E(3,1,1))
     &            ULY = E(4,1,1)
               DO 2 Y = LLY,ULY
                  LLZ = MAX0(0,K-V-W-X-Y)
                  ULZ = MIN0(R5,C1-V-W-X-Y)
                  Z1  = LLZ
                  YY  = ONE*Y
                  ZZ1 = ONE*Z1
                  DO 1 Z = LLZ,ULZ
                     IF (V.EQ.E(1,1,1).AND.W.EQ.E(2,1,1).AND.X.
     &                  EQ.E(3,1,1).AND.Y.EQ.E(4,1,1).AND.Z.EQ.
     &                  E(5,1,1)) THEN
                        CRIT = H*(ONE+EPSLN)
                        GO TO 6
                     END IF
                     ZZ = ONE*Z
                     H  = H*(RR5-ZZ)*(CC1-VV-WW-XX-YY-ZZ)/(ZZ+ONE)/
     &                    (LL+VV+WW+XX+YY+ZZ)
    1             CONTINUE
                  IF (Z1.EQ.0) THEN
                     H1 = H1*(RR4-YY)*(CC1-VV-WW-XX-YY)/(YY+ONE)/
     &                    (LL+VV+WW+XX+YY)
                  ELSE
                     H1 = H1*(RR4-YY)*ZZ1/(YY+ONE)/(RR5-ZZ1+ONE)
                  END IF
                  H = H1
    2          CONTINUE
               IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
                  H2 = H2*(RR3-XX)*(CC1-VV-WW-XX)/(XX+ONE)/(LL+VV+WW+XX)
               ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
                  H2 = H2*(RR3-XX)*ZZ2/(XX+ONE)/(RR5-ZZ2+ONE)
               ELSE
                  H2 = H2*(RR3-XX)*YY2/(XX+ONE)/(RR4-YY2+ONE)
               END IF
               H  = H2
               H1 = H2
    3       CONTINUE
            IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
               H3 = H3*(RR2-WW)*(CC1-VV-WW)/(WW+ONE)/(LL+VV+WW)
            ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
               H3 = H3*(RR2-WW)*ZZ3/(WW+ONE)/(RR5-ZZ3+ONE)
            ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
               H3 = H3*(RR2-WW)*YY3/(WW+ONE)/(RR4-YY3+ONE)
            ELSE
               H3 = H3*(RR2-WW)*XX3/(WW+ONE)/(RR3-XX3+ONE)
            END IF
            H  = H3
            H1 = H3
            H2 = H3
    4    CONTINUE
         IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.EQ.0.AND.Z4.EQ.0) THEN
            H4 = H4*(RR1-VV)*(CC1-VV)/(VV+ONE)/(LL+VV)
         ELSE IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.EQ.0.AND.Z4.GT.0) THEN
            H4 = H4*(RR1-VV)*ZZ4/(VV+ONE)/(RR5-ZZ4+ONE)
         ELSE IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.GT.0) THEN
            H4 = H4*(RR1-VV)*YY4/(VV+ONE)/(RR4-YY4+ONE)
         ELSE IF (W4.EQ.0.AND.X4.GT.0) THEN
            H4 = H4*(RR1-VV)*XX4/(VV+ONE)/(RR3-XX4+ONE)
         ELSE
            H4 = H4*(RR1-VV)*WW4/(VV+ONE)/(RR2-WW4+ONE)
         END IF
         H  = H4
         H1 = H4
         H2 = H4
         H3 = H4
    5 CONTINUE
    6 H  = INIT
      H1 = INIT
      H2 = INIT
      H3 = INIT
      H4 = INIT
      ITAB = 0
      ITAIL = 0
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 11 V = LLV,ULV
         LLW = MAX0(0,K-R3-R4-R5-V)
         ULW = MIN0(R2,C1-V)
         W4  = LLW
         X4  = MAX0(0,K-R4-R5-V-W4)
         Y4  = MAX0(0,K-R5-V-W4-X4)
         Z4  = MAX0(0,K-V-W4-X4-Y4)
         VV  = ONE*V
         WW4 = ONE*W4
         XX4 = ONE*X4
         YY4 = ONE*Y4
         ZZ4 = ONE*Z4
         DO 10 W = LLW,ULW
            LLX = MAX0(0,K-R4-R5-V-W)
            ULX = MIN0(R3,C1-V-W)
            X3  = LLX
            Y3  = MAX0(0,K-R5-V-W-X3)
            Z3  = MAX0(0,K-V-W-X3-Y3)
            WW  = ONE*W
            XX3 = ONE*X3
            YY3 = ONE*Y3
            ZZ3 = ONE*Z3
            DO 9 X = LLX,ULX
               LLY = MAX0(0,K-R5-V-W-X)
               ULY = MIN0(R4,C1-V-W-X)
               Y2  = LLY
               Z2  = MAX0(0,K-V-W-X-Y2)
               XX  = ONE*X
               YY2 = ONE*Y2
               ZZ2 = ONE*Z2
               DO 8 Y = LLY,ULY
                  LLZ = MAX0(0,K-V-W-X-Y)
                  ULZ = MIN0(R5,C1-V-W-X-Y)
                  Z1  = LLZ
                  YY  = ONE*Y
                  ZZ1 = ONE*Z1
                  DO 7 Z = LLZ,ULZ
                     ITAB = ITAB+1
                     TOT = TOT+H
                     IF (H.LE.CRIT) THEN
                        ITAIL = ITAIL+1
                        SUM = SUM+H
                     END IF
                     ZZ = ONE*Z
                     H  = H*(RR5-ZZ)*(CC1-VV-WW-XX-YY-ZZ)/(ZZ+ONE)/
     &                    (LL+VV+WW+XX+YY+ZZ)
    7             CONTINUE
                  IF (Z1.EQ.0) THEN
                     H1 = H1*(RR4-YY)*(CC1-VV-WW-XX-YY)/(YY+ONE)/
     &                    (LL+VV+WW+XX+YY)
                  ELSE
                     H1 = H1*(RR4-YY)*ZZ1/(YY+ONE)/(RR5-ZZ1+ONE)
                  END IF
                  H = H1
    8          CONTINUE
               IF (Y2.EQ.0.AND.Z2.EQ.0) THEN
                  H2 = H2*(RR3-XX)*(CC1-VV-WW-XX)/(XX+ONE)/(LL+VV+WW+XX)
               ELSE IF (Y2.EQ.0.AND.Z2.GT.0) THEN
                  H2 = H2*(RR3-XX)*ZZ2/(XX+ONE)/(RR5-ZZ2+ONE)
               ELSE
                  H2 = H2*(RR3-XX)*YY2/(XX+ONE)/(RR4-YY2+ONE)
               END IF
               H  = H2
               H1 = H2
    9       CONTINUE
            IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.EQ.0) THEN
               H3 = H3*(RR2-WW)*(CC1-VV-WW)/(WW+ONE)/(LL+VV+WW)
            ELSE IF (X3.EQ.0.AND.Y3.EQ.0.AND.Z3.GT.0) THEN
               H3 = H3*(RR2-WW)*ZZ3/(WW+ONE)/(RR5-ZZ3+ONE)
            ELSE IF (X3.EQ.0.AND.Y3.GT.0) THEN
               H3 = H3*(RR2-WW)*YY3/(WW+ONE)/(RR4-YY3+ONE)
            ELSE
               H3 = H3*(RR2-WW)*XX3/(WW+ONE)/(RR3-XX3+ONE)
            END IF
            H  = H3
            H1 = H3
            H2 = H3
   10    CONTINUE
         IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.EQ.0.AND.Z4.EQ.0) THEN
            H4 = H4*(RR1-VV)*(CC1-VV)/(VV+ONE)/(LL+VV)
         ELSE IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.EQ.0.AND.Z4.GT.0) THEN
            H4 = H4*(RR1-VV)*ZZ4/(VV+ONE)/(RR5-ZZ4+ONE)
         ELSE IF (W4.EQ.0.AND.X4.EQ.0.AND.Y4.GT.0) THEN
            H4 = H4*(RR1-VV)*YY4/(VV+ONE)/(RR4-YY4+ONE)
         ELSE IF (W4.EQ.0.AND.X4.GT.0) THEN
            H4 = H4*(RR1-VV)*XX4/(VV+ONE)/(RR3-XX4+ONE)
         ELSE
            H4 = H4*(RR1-VV)*WW4/(VV+ONE)/(RR2-WW4+ONE)
         END IF
         H  = H4
         H1 = H4
         H2 = H4
         H3 = H4
   11 CONTINUE
C
C     CALCULATE THE PROBABILITY VALUE FOR THE 6 BY 2
C     CROSS-CLASSIFICATION TABLE
C
      P = SUM/TOT
      PX = CRIT/TOT
      RETURN
      END
