      PROGRAM M13
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM M13 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE FISHER'S EXACT PROBABILITY FOR
C     13 CELL GOODNESS-OF-FIT TABLES.
C
      DIMENSION  L(13),P(13),PL(13)
C
C     COMPUTE FISHER'S EXACT TEST FOR 13 CELL G-O-F TABLES
C
      WRITE(*,2) 'FISHER''S EXACT TEST FOR SPARSE 13 CELL G-O-F TABLES'
    2 FORMAT(/,T2,A,/)
      WRITE(*,3) 'ENTER L1, L2, L3, L4, L5, L6, L7, L8, L9, L10, L11,',
     +           ' L12 & L13:'
    3 FORMAT(T2,A,A)
      READ(*,*)  L(1),L(2),L(3),L(4),L(5),L(6),L(7),L(8),L(9),L(10),
     +           L(11),L(12),L(13)
      WRITE(*,3) 'ENTER P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11',
     +           ' & P12:'
      READ(*,*)  P(1),P(2),P(3),P(4),P(5),P(6),P(7),P(8),P(9),P(10),
     +           P(11),P(12)
      P(13) = 1.0D0-P(1)-P(2)-P(3)-P(4)-P(5)-P(6)-P(7)-P(8)-P(9)-P(10)
     +        -P(11)-P(12)
      N = 0
      DO 4 I = 1,13
           PL(I) = DLOG(P(I))
           N = N+L(I)
    4 CONTINUE
      WRITE(*,5)
    5 FORMAT(T15,'FISHER''S EXACT TEST FOR 13 CELL G-O-F TABLES',/)
      WRITE(*,6) '13 OBSERVED CELL VALUES: ',L(1),L(2),L(3),L(4),L(5),
     +   L(6),L(7),L(8),L(9),L(10),L(11),L(12),L(13)
    6 FORMAT(T5,A,13I3)
      WRITE(*,7) 'TOTAL OF ALL 13 CELL VALUES: ',N
    7 FORMAT(T5,A,I12)
      CALL FE13(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +          IFE,ICS,IZS,ILR,CHECK)
      WRITE(*,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE(*,7) '# OF TABLES YIELDING FISHER''S EXACT P-VALUE = ',IFE
      WRITE(*,8) 'P(OBSERVED TABLE) = ',PT
    8 FORMAT(T5,A,D16.9)
      WRITE(*,8) 'FISHER''S EXACT 13 CELL G-O-F TEST P-VALUE = ',PFE
      WRITE(*,7) '# OF TABLES YIELDING EXACT CHI-SQUARED P-VALUE = ',ICS
      IF(CST.LT.0.0D0) CST = 0.0D0
      WRITE(*,8) 'OBSERVED CHI-SQUARED STATISTIC = ',CST
      WRITE(*,8) 'EXACT CHI-SQUARED 13 CELL G-O-F TEST P-VALUE = ',PCS
      WRITE(*,7) '# OF TABLES YIELDING EXACT ZELTERMAN P-VALUE = ',IZS
      IF(CZT.LT.0.0D0) CZT = 0.0D0
      WRITE(*,8) 'OBSERVED ZELTERMAN STATISTIC = ',CZT
      WRITE(*,8) 'EXACT ZELTERMAN 13 CELL G-O-F TEST P-VALUE = ',PZS
      WRITE(*,7) '# OF TABLES YIELDING EXACT LR TEST P-VALUE = ',ILR
      IF(TLR.LT.0.0D0) TLR = 0.0D0
      WRITE(*,8) 'OBSERVED LR TEST STATISTIC = ',TLR
      WRITE(*,8) 'EXACT LR TEST 13 CELL G-O-F TEST P-VALUE = ',PLR
      WRITE(*,8) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE FE13(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +                IFE,ICS,IZS,ILR,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  A FISHER'S EXACT TEST FOR TESTING GOODNESS-OF-FIT FOR 13 CELLS.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PFE   -- OUTPUT, FISHER'S EXACT PROBABILITY VALUE (P-VALUE)
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        IFE   -- OUTPUT, # OF TABLES COMPRISING FISHER'S EXACT P-VALUE
C        CHECK -- OUTPUT, SUM OF ALL CONFIGURATION PROBABILITIES
C     USAGE:
C        SUBROUTINE FE13 COMPUTES THE FISHER EXACT PROBABILITY TEST
C        FOR THE GOODNESS-OF-FIT OF 13 OBSERVED CELL FREQUENCIES
C
      DIMENSION L(13),P(13),PL(13)
C
      Z = FAC(N)
C  DETERMINE THE PROBABILITY OF THE OBSERVED 13 CELL FREQUENCIES.
      THISP = PR(Z,PL,L)*(1.0D0-1.0D-12)
      CST = CS(N,P,L)*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CZT = CZ(N,P,L)*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      TLR = ZL(N,P,L)*(1.0D0-1.0D-12)
      IF(TLR.LT.1.0D-11) TLR = -1.0D-11
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      ITAB = 0
      IFE = 0
      ICS = 0
      IZS = 0
      ILR = 0
      PFE = 0.0D0
      PCS = 0.0D0
      PZS = 0.0D0
      PLR = 0.0D0
      CHECK = 0.0D0
C  12 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 12 SPECIFIED
C  CELLS (THE REMAINING CELL IS OBTAINED FROM THE 12 SPECIFIED CELLS).
      DO 40 I1 = 0,N
      M2 = N-I1
      DO 39 I2 = 0,M2
      M3 = N-I1-I2
      DO 38 I3 = 0,M3
      M4 = N-I1-I2-I3
      DO 37 I4 = 0,M4
      M5 = N-I1-I2-I3-I4
      DO 36 I5 = 0,M5
      M6 = N-I1-I2-I3-I4-I5
      DO 35 I6 = 0,M6
      M7 = N-I1-I2-I3-I4-I5-I6
      DO 34 I7 = 0,M7
      M8 = N-I1-I2-I3-I4-I5-I6-I7
      DO 33 I8 = 0,M8
      M9 = N-I1-I2-I3-I4-I5-I6-I7-I8
      DO 32 I9 = 0,M9
      M10 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9
      DO 31 I10 = 0,M10
      M11 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10
      DO 30 I11 = 0,M11
      M12 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11
      DO 29 I12 = 0,M12
      L(1) = I1
      L(2) = I2
      L(3) = I3
      L(4) = I4
      L(5) = I5
      L(6) = I6
      L(7) = I7
      L(8) = I8
      L(9) = I9
      L(10) = I10
      L(11) = I11
      L(12) = I12
      L(13) = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12
C
       ITAB = ITAB+1
       CP = PR(Z,PL,L)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       IF(CP.LT.THISP) THEN
        IFE = IFE+1
        PFE = PFE+EXCP
       ENDIF
       CC = CS(N,P,L)
       IF(CC.GT.CST) THEN
        ICS = ICS+1
        PCS = PCS+EXCP
       ENDIF
       U = CZ(N,P,L)
       IF(U.GT.CZT) THEN
        IZS = IZS+1
        PZS = PZS+EXCP
       ENDIF
       W = ZL(N,P,L)
       IF(W.GT.TLR) THEN
        ILR = ILR+1
        PLR = PLR+EXCP
       ENDIF
   29 CONTINUE
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
   33 CONTINUE
   34 CONTINUE
   35 CONTINUE
   36 CONTINUE
   37 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      PT = DEXP(THISP)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION PR(Z,PL,L)
C  OBTAIN SUM OF LOG PROBABILITY FOR 13 CELLS.
      DOUBLE PRECISION  Z,FAC,PL(13)
      INTEGER  I,L(13)
      PR = Z
      DO 10 I = 1,13
         PR = PR-FAC(L(I))+L(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CS(N,P,L)
C  OBTAIN CHI-SQUARED STATISTIC
      DOUBLE PRECISION  ONE,P(13)
      INTEGER  I,L(13)
      ONE = 1.0D0
      CS = 0.0D0
      DO 10 I = 1,13
         CS = CS+(ONE*L(I))**2/P(I)
   10 CONTINUE
      CS = CS/N-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CZ(N,P,L)
C  OBTAIN ZELTERMAN STATISTIC
      DOUBLE PRECISION  ONE,P(13)
      INTEGER  I,L(13)
      ONE = 1.0D0
      CZ = 0.0D0
      DO 10 I = 1,13
         CZ = CZ+ONE*L(I)*(L(I)-1)/P(I)
   10 CONTINUE
      CZ = CZ/N+13-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION ZL(N,P,L)
C  OBTAIN LR TEST STATISTIC
      DOUBLE PRECISION  X,RZ,P(13)
      INTEGER  I,L(13)
      ZL = 0.0D0
      DO 10 I = 1,13
         X = 1.0D0*L(I)
         IF(X.GT.1.5D0) THEN
            RZ = X*DLOG(X)
         ELSE
            RZ = 0.0D0
         ENDIF
         ZL = ZL+RZ-DLOG(P(I)*N)*L(I)
   10 CONTINUE
      ZL = ZL*2
      RETURN
      END
