      PROGRAM R2S4
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM R2S4 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE EXACT COMPARISON-OF-TWO-SAMPLES
C     RIDIT ANALYSIS LOWER AND UPPER TAIL P-VALUES FOR A
C     4 CELL TABLE.
C
      DIMENSION  K(4),L(4),PL(4),R(4),S(4)
      CHARACTER  FOUT*40
C
      WRITE(*,*)  'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,1) FOUT
    1 FORMAT(A)
      ONE = 1.0D0
      WRITE(*,2) 'EXACT TWO-SAMPLE RIDIT ANALYSIS FOR 4 CELL TABLES'
    2 FORMAT(T2,A)
      WRITE(*,3) 'ENTER FREQUENCIES K1, K2, K3 & K4 OF SAMPLE 1:'
    3 FORMAT(T2,A)
      READ(*,*)  K(1),K(2),K(3),K(4)
      WRITE(*,3) 'ENTER FREQUENCIES L1, L2, L3 & L4 OF SAMPLE 2:'
      READ(*,*)  L(1),L(2),L(3),L(4)
      N1 = 0
      N2 = 0
      DO 4 I = 1,4
           N1 = N1+K(I)
           N2 = N2+L(I)
    4 CONTINUE
      R(1) = ONE*K(1)/(2*N1)
      R(2) = ONE*K(1)/N1+ONE*K(2)/(2*N1)
      R(3) = ONE*(K(1)+K(2))/N1+ONE*K(3)/(2*N1)
      R(4) = ONE*(K(1)+K(2)+K(3))/N1+ONE*K(4)/(2*N1)
      S(1) = ONE*L(1)/(2*N2)
      S(2) = ONE*L(1)/N2+ONE*L(2)/(2*N2)
      S(3) = ONE*(L(1)+L(2))/N2+ONE*L(3)/(2*N2)
      S(4) = ONE*(L(1)+L(2)+L(3))/N2+ONE*L(4)/(2*N2)
      X1 = 0.0D0
      X2 = 0.0D0
      DO 5 I = 1,4
           PL(I) = DLOG(0.25D0)
           X1 = X1+S(I)*K(I)
           X2 = X2+R(I)*L(I)
    5 CONTINUE
      X1 = X1/N1
      X2 = X2/N2
      OPEN(2,FILE=FOUT,STATUS='NEW')
      WRITE(2,6)
    6 FORMAT(T15,'EXACT TWO-SAMPLE RIDIT ANALYSIS FOR 4 CELL TABLES')
      WRITE(2,7) '4 SAMPLE 1 CELL FREQUENCIES: ',K(1),K(2),K(3),K(4)
    7 FORMAT(T5,A,4I6)
      WRITE(2,7) '4 SAMPLE 2 CELL FREQUENCIES: ',L(1),L(2),L(3),L(4)
      WRITE(2,8) 'TOTAL OF 4 SAMPLE 1 CELL FREQUENCIES: ',N1
    8 FORMAT(T5,A,I12)
      WRITE(2,8) 'TOTAL OF 4 SAMPLE 2 CELL FREQUENCIES: ',N2
      XT = X1-X2
      CALL ER4(K,L,PL,R,S,N1,N2,PT,XT,PERL,PERU,ITAB,IERL,IERU,
     +         CHECK,Y1,Y2)
      SE = DSQRT(Y2-Y1**2)
      W = (XT-Y1)/SE
      CALL NORMAL(W,PNL,PNU)
      WRITE(2,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE(2,8) '# OF TABLES IN THE EXACT LOWER TAIL P-VALUE = ',IERL
      WRITE(2,8) '# OF TABLES IN THE EXACT UPPER TAIL P-VALUE = ',IERU
      WRITE(2,9) 'P(OBSERVED TABLE) = ',PT
    9 FORMAT(T5,A,D16.9)
      WRITE(2,9) 'OBSERVED SAMPLE 1 RIDIT STATISTIC = ',X1
      WRITE(2,9) 'OBSERVED SAMPLE 2 RIDIT STATISTIC = ',X2
      WRITE(2,9) 'OBSERVED TWO-SAMPLE RIDIT STATISTIC = ',XT
      WRITE(2,9) 'EXACT EXPECTED VALUE OF RIDIT STATISTIC = ',Y1
      WRITE(2,9) 'EXACT STANDARD ERROR OF RIDIT STATISTIC = ',SE
      WRITE(2,9) 'EXACT STANDARDIZED NORMAL TEST STATISTIC = ',W
      WRITE(2,9) 'NORMAL LOWER TAIL P-VALUE FOR 4 CELL TABLE = ',PNL
      WRITE(2,9) 'NORMAL UPPER TAIL P-VALUE FOR 4 CELL TABLE = ',PNU
      WRITE(2,9) 'EXACT LOWER TAIL P-VALUE FOR 4 CELL TABLE = ',PERL
      WRITE(2,9) 'EXACT UPPER TAIL P-VALUE FOR 4 CELL TABLE = ',PERU
      WRITE(2,9) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE ER4(K,L,PL,R,S,N1,N2,PT,WX,PERL,PERU,ITAB,IERL,IERU,
     +               CHECK,Y1,Y2)
      IMPLICIT REAL*8(A-H,O-Z)
C
C   EXACT TEST FOR RIDIT ANALYSIS OF 4 CELL TABLE.
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C     K     -- INPUT,  ARRAY CONTAINING SAMPLE 1 CELL FREQUENCIES
C     L     -- INPUT,  ARRAY CONTAINING SAMPLE 2 CELL FREQUENCIES
C     PL    -- INPUT,  ARRAY CONTAINING THE CELL PROBABILITY LOGARITHMS
C     R     -- INPUT,  ARRAY CONTAINING OBSERVED SAMPLE 1 RIDITS
C     S     -- INPUT,  ARRAY CONTAINING OBSERVED SAMPLE 2 RIDITS
C     PERL  -- OUTPUT, EXACT RIDIT ANALYSIS LOWER-TAIL P-VALUE
C     PERU  -- OUTPUT, EXACT RIDIT ANALYSIS UPPER-TAIL P-VALUE
C     PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C     ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C     IERL  -- OUTPUT, # OF TABLES COMPRISING LOWER-TAIL RIDIT ANALYSIS P-VALUE
C     IERU  -- OUTPUT, # OF TABLES COMPRISING UPPER-TAIL RIDIT ANALYSIS P-VALUE
C     CHECK -- OUTPUT, SUM OF ALL CONFIGURATION PROBABILITIES
C
C     USAGE:
C        SUBROUTINE ER4 COMPUTES THE EXACT AND APPROXIMATE TWO-SAMPLE RIDIT
C        ANALYSIS P-VALUES FOR TESTING ORDERED PECULIARITY OF A 4 CELL TABLE
C
      DIMENSION K(4),L(4),PL(4),R(4),S(4)
C
      Z1 = FAC(N1)
      Z2 = FAC(N2)
      PT = DEXP(PR(Z1,PL,K)+PR(Z2,PL,L))
C  DETERMINE THE PROBABILITY OF THE OBSERVED 4 CELL FREQUENCIES.
      WL = WX*(1.0D0+1.0D-12)
      WU = WX*(1.0D0-1.0D-12)
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      ITAB = 0
      IERL = 0
      IERU = 0
      PERL = 0.0D0
      PERU = 0.0D0
      CHECK = 0.0D0
      Y1 = 0.0D0
      Y2 = 0.0D0
C  TWO SETS OF 3 MAJOR LOOPS TO DETERMINE THE VALID TABLES FOR TWO-SAMPLE
C  RIDIT ANALYSIS FROM ALL VALUES OF 3 SPECIFIED CELLS ASSOCIATED WITH EACH
C  SAMPLE (REMAINING CELLS ARE OBTAINED FROM EACH SAMPLE'S 3 SPECIFIED CELLS).
      DO 40 I1 = 0,N1
      MA2 = N1-I1
      DO 39 I2 = 0,MA2
      MA3 = N1-I1-I2
      DO 38 I3 = 0,MA3
       K(1) = I1
       K(2) = I2
       K(3) = I3
       K(4) = N1-I1-I2-I3
       X1 = 0.0D0
       DO 21 I = 1,4
           X1 = X1+S(I)*K(I)
   21  CONTINUE
       X1 = X1/N1
      DO 30 J1 = 0,N2
      MB2 = N2-J1
      DO 29 J2 = 0,MB2
      MB3 = N2-J1-J2
      DO 28 J3 = 0,MB3
       L(1) = J1
       L(2) = J2
       L(3) = J3
       L(4) = N2-J1-J2-J3
       X2 = 0.0D0
       DO 22 I = 1,4
           X2 = X2+R(I)*L(I)
   22  CONTINUE
       X2 = X2/N2
C
       ITAB = ITAB+1
       CP = PR(Z1,PL,K)+PR(Z2,PL,L)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       W = X1-X2
       Y1 = Y1+EXCP*W
       Y2 = Y2+EXCP*W**2
       IF(W.LT.WL) THEN
        IERL = IERL+1
        PERL = PERL+EXCP
       ENDIF
       IF(W.GT.WU) THEN
        IERU = IERU+1
        PERU = PERU+EXCP
       ENDIF
   28 CONTINUE
   29 CONTINUE
   30 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION PR(Z,PL,L)
C  OBTAIN SUM OF LOG PROBABILITY FOR 4 CELLS.
      DOUBLE PRECISION  Z,FAC,PL(4)
      INTEGER  I,L(4)
      PR = Z
      DO 10 I = 1,4
         PR = PR-FAC(L(I))+L(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C  ...........................................................................
      SUBROUTINE NORMAL(Z,PNL,PNU)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PNL,PNU,PROB,ZERO,ONE,PI,G,A,B,C,E,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.GT.ZERO) THEN
         PNU = PROB
         PNL = ONE-PROB
      ELSE
         PNL = PROB
         PNU = ONE-PROB
      END IF
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
